<?php
/*****************************
* Copyright 2021 1st Source IT, LLC
* All rights reserved.
* Permission granted for use as
* long as this copyright notice, associated text and
* links remain in tact.
* Licensed for a single domain and a single instance of EZ-cart.
* Additional licenses can be purchased for additonal sites.
*
* http://www.ez-ms.com
* http://www.ez-om.com
* http://www.1sit.com*
*
* End copyright notification
*/
if( !defined('BOOTSTRAP') ) die('Access denied');

use Tygh\Registry;

// Abandoned carts...
function am_abandon_age_ts($val) {
	$age = strtolower($val);
	$components = preg_split('#(?<=\d)(?=[a-z])#i', $age);
	
	// Use first 2.
	$measure = $components[1];
	$numeric = $components[0];
	
	$factor = 4 * (SECONDS_IN_DAY/24);	// 4hr
	if( $components && count($components) > 1 ) {
		switch($measure) {
			case 'h':
			case 'hr':
			case 'hrs':
			case 'hour':
				$factor = 60 * 60;
				break;	// default factor
			case 'm':
			case 'min':
			case 'mins':
			case 'minute':
				$factor = 60;
				break;
			case 'd':
			case 'day':
			case 'days':
				$factor = SECONDS_IN_DAY;
				break;
			case 'w':
			case 'wk':
			case 'week':
			case 'weeks':
				$factor = SECONDS_IN_DAY * 7;
				break;
			case 'mth':
			case 'month':
			case 'months':
				$factor = SECONDS_IN_DAY * 30;
				break;
		}
	}
	return $numeric * $factor;
}

function am_event_abandoned_carts() {
	$company_id = am_company_id();
	foreach(am_get_events(CART_EVENT, 'A', $company_id, true) as $event_id => $event_info) {
		$am_age = $event_info['am_event_options']['abandon_age'];
		foreach(am_abandoned_carts($am_age) as $cart) {
			am_dispatch_cart_event($cart, $am_age);
		}
	}
}

function am_session_cart($session_id) {
	$c = array();
	
	// Ages already processed
	foreach(db_get_array("SELECT * FROM ?:user_session_products WHERE session_id=?s", $session_id) as $p) {
		
		$item_id = $p['item_id'];
		if( !empty($p['extra']) )
			$p['extra'] = unserialize($p['extra']);
			
		// Initialize the non-product elements of the cart data
		if( empty($c) ) {
			$c = $p;
			$c['products'] = array();
			
			// User data
			$u_data = fn_get_user_info($p['user_id']);
			if( $u_data) {
				unset($u_data['password'], $u_data['salt'], $u_data['sq_data']);
			}
			$c['user_data'] = $u_data;
			if(empty($c['user_data']['email']) && !empty($p['extra']['email']) ) {
				$c['user_data']['email'] = $p['extra']['email'];
			}
			$c['incomplete'] = empty($p['order_id']) ? $p['order_id'] : '';
			$c['am_ages'] = empty($p['am_ages']) ? array() : explode(",", $p['am_ages']);
			
			unset($c['extra'], $c['item_id'], $c['item_type'], 
					$c['product_id'], $c['amount'], $c['price']
				  );
		} 
		
		$c['products'][$item_id] = $p['extra'];
	}	// foreach
	return $c;
}

function am_abandoned_carts($age) {
	$age_ts = am_abandon_age_ts($age);

	$condition = sprintf("timestamp <= %d", (TIME - $age_ts) );
	$age_condition = sprintf("AND am_ages != '%s' AND FIND_IN_SET('%s', am_ages)=0", 
										AM_CART_BEFORE_INSTALL, $age );
	
	$session_ids = db_get_fields($q="SELECT DISTINCT(session_id) FROM ?:user_session_products WHERE $condition $age_condition ORDER BY timestamp DESC");
	$carts = array();
	foreach($session_ids as $session_id) {
		$cart = am_session_cart($session_id);
		if( !empty($cart) && !empty($cart['user_data']['email']) ) {
			$carts[$session_id] = $cart;
		}
	}	// foreach
	return $carts;
}		

function am_update_ages($age, $session_id) {
	$q = "UPDATE ?:user_session_products SET am_ages=if( am_ages='', ?s, concat(',', ?s))
			WHERE session_id=?s";
	db_query($q, $age, $age, $session_id);
}

?>