<?php
/*****************************
* Copyright 2021 1st Source IT, LLC
* All rights reserved.
* Permission granted for use as
* long as this copyright notice, associated text and
* links remain in tact.
* Licensed for a single domain and a single instance of EZ-cart.
* Additional licenses can be purchased for additonal sites.
*
* http://www.ez-ms.com
* http://www.ez-om.com
* http://www.1sit.com*
*
* End copyright notification
*/
if ( !defined('BOOTSTRAP') ) { die('Access denied'); }
use Tygh\Registry;
use Tygh\Settings;
use Tygh\Mailer;

// Compatibility
$dir_addons = Registry::get('config.dir.addons');
$index_script = Registry::get('config.admin_index');

$v = explode(DIRECTORY_SEPARATOR, __FILE__);
$addon_name = $v[count($v)-4];


/* Every addon must have this to bootstrap the ez_common addon
 */
if( file_exists($f = ez_fix_path($dir_addons."ez_common/lib/upgrade.php")) ) {
	require_once($f);
} else {	// Need to force update of the tools
	require_once(ez_fix_path($dir_addons."$controller/lib/upgrade/upgrade.php"));		// copy of common version of tools - used to bootstrap the ez_common addon
}

switch($mode) {
	case 'log':
		if( class_exists('ez_log') ) {
			$log_lines = 10;
			$l_buf = array();
			switch($action) {
				case 'clear': 
					ez_log::clear_log($addon_name); 
					fn_set_notification('N', $controller, "Cleared log file");
					break;
				case 'truncate':
					$truncate_lines = $lines = (empty($_REQUEST['truncate_lines']) && $dispatch_extra) 
								? $dispatch_extra 
								: (empty($_REQUEST['truncate_lines']) ? 100 : $_REQUEST['truncate_lines']);
					$entries = ez_log::truncate_log($addon_name, $lines);
					fn_set_notification('N', $controller, "Truncated log to $entries lines");
					break;
				case 'show':
					$lines = min($log_lines, (empty($_REQUEST['show_lines']) && $dispatch_extra) 
								? $dispatch_extra 
								: (empty($_REQUEST['show_lines']) ? 100 : $_REQUEST['show_lines']));
					ez_log::show_log($addon_name, $lines);
					exit;
				case 'set_debug':
					$level = $_REQUEST['debug_level'];
					am_set_debug_level($level);
					Registry::get('view')->assign('debug_level', $level);
					am_log("$controller.$mode.$action: Set debug level to '$level'");
					break;
			}	// switch
			if( !empty($action) ) 
				fn_redirect("$controller.$mode");
		}	// class exists

		// Default is the get method. 
		$show_lines = $lines = $dispatch_extra ? $dispatch_extra : (empty($_REQUEST['show_lines']) ? 100 : $_REQUEST['show_lines']);
		if( $_SERVER['REQUEST_METHOD'] != 'POST') {
			// 2/8/14: Add check for ez_log class
			$l_buf = class_exists('ez_log') ? ez_log::get_log_entries('auto_mail', $lines) : array();;
			$log_lines = count($l_buf);
			$dat = @file_get_contents(auto_mail_dbg_logfile);
			if( $dat ) {
				$l_buf = explode(auto_mail_dbg_end_record, $dat);
				$log_lines = count($l_buf);
			}
			// But fwe may have a minimum number of lines in the log
			$lines = min($log_lines, (empty($_REQUEST['show_lines']) && $dispatch_extra) 
				? $dispatch_extra 
				: (empty($_REQUEST['show_lines']) ? 100 : $_REQUEST['show_lines']));
		}
		Registry::get('view')->assign('log_lines', $log_lines);
		// 2/8/14 added check of ez_class exists
		Registry::get('view')->assign('debug_level', class_exists('ez_log') ? ez_log::get_log_level('auto_mail') : 0);
//		Registry::get('view')->assign('debug_level', am_get_debug_level());
		Registry::get('view')->assign('debug_menu', am_debug_menu());
		Registry::get('view')->assign('show_lines', $show_lines);
		Registry::get('view')->assign('truncate_lines', $show_lines);
		
		if( $_SERVER['REQUEST_METHOD'] == 'POST' ) {
			return array(CONTROLLER_STATUS_REDIRECT, "$controller.$mode");
		} else {
			// Show the log for all get reqeusts.
			am_show_log($l_buf, $log_lines, $lines);
			return array(CONTROLLER_STATUS_OK);
		}
		break;
	case 'set_default_events':	// Test
		include($dir_addons."auto_mail/tests/set_events.php");
		foreach(am_event_types() as $event_type) {
			$d = am_get_events($event_type);
			am_dbg("$mode: type=$event_type:".print_r($d,true), true);
		}
		exit;

	case 'check_install':
		if( file_exists($f=ez_fix_path(Registry::get('config.dir.addons')."$controller/upgrade/post_upgrade_all_versions.php")) )
			include($f);
		include(Registry::get('config.dir.addons')."$controller/config.php");
		if( file_exists($f = ez_fix_path(sprintf("%s$controller/upgrade/post_upgrade_%s.php", Registry::get('config.dir.addons'), $addon_cur_ver)) ) )
			include($f);
									 
		$result = auto_mail_install_setup();
		am_check_table_fields();
		if( $result === false )
			$result = "Failed";
		else $result = "Okay";
		fn_set_notification($result == "Failed" ? 'E' : 'N', "$controller.$mode", $result, 'K');
		fn_redirect("$controller.manage");

	case 'show_statuses':
		am_dbg("statuses:".print_r(am_order_status_menu(),true), true);
		exit;
	case 'test_order':
		$order_id = $action;
		$o = fn_get_order_info($order_id);
//		$o['products'] =& $o['items'];
//am_dbg("$mode: o:".print_r($o,true), true);
		$queued_data = am_dispatch_order_event($order_id, $o, 'P');
//		am_dbg("$mode: queued_data:".print_r($queued_data,true));
		fn_redirect("auto_mail.log");
		exit;
	case 'manage':
		if( $_SERVER['REQUEST_METHOD'] == 'POST' ) {
			$am_events = empty($_REQUEST['am_events']) ? array() : $_REQUEST['am_events'];
			foreach($am_events as $am_event_type => $e_data) {
				foreach($e_data as $event_id => $e_info) {
					$e_info['company_id'] = am_company_id();
					am_update_event($event_id, $e_info);
				}
			}
			fn_redirect("$controller.$mode");
		}
		$am_events = array();
		foreach(am_event_types() as $event_type)
			$am_events[$event_type] = am_get_events($event_type, '', am_company_id(), true);
		Registry::get('view')->assign('am_events', $am_events);
		$x = am_template_menu("");
		// Strip lang_code/ and .tpl from template name
		$preview_templates = array();
		foreach($x as $template_name => $desc) {
			$new_name = str_replace('.tpl', '', $template_name);
			$preview_templates[$new_name] = $desc;
		}
		Registry::get('view')->assign('preview_template_menu', $preview_templates );
		Registry::get('view')->assign('unsubscribed_count', (integer)db_get_field("SELECT count(email) FROM ?:auto_mail_subscribers WHERE status='N'"));
//		Registry::get('view')->assign('last_queue_id', db_get_field("SELECT am_queue_id from ?:auto_mail_queue WHERE am_event_type='O' ORDER BY am_queue_id DESC LIMIT 1"));
//foreach($am_events as $event_type =>$events)
// foreach($events as $e_data)
//	am_dbg("$mode: am_event_name='$e_data[am_event_name]'", true);
//am_dbg("$mode: am_events:".print_r($am_events,true), true);
		return array(CONTROLLER_STATUS_OK);
		break;
	case 'edit_event':
		if( $_SERVER['REQUEST_METHOD'] == 'POST' && !empty($_REQUEST['event']) ) {
			$_event = $_REQUEST['event'];
			$_action_data = $_event['am_action_data'];
			unset($_event['am_action_data']);
			if( empty($_event['am_event_id']) )
				unset($_event['am_event_id']);
			$_event['am_action_data'] = array();
			switch($action_type = $_event['am_action_type']) {
				case ORDER_EVENT_PLACED:
				case ORDER_EVENT_AMOUNT_GREATER:
				case ORDER_EVENT_PAYMENT_TYPE:
				case ORDER_EVENT_SHIP_METHOD:
				case PROFILE_EVENT_ADD:
				case CART_EVENT_ABANDONED:
				case FORM_EVENT:
					$_event['am_action_data'] = $_action_data[$action_type];
					break;
				case ORDER_EVENT_PRODUCTS_PURCHASED:	// comma list
					$x = explode(',', $_action_data[$action_type]);
					foreach($x as $k => $v)
						$x[$k] = trim($v);	// get rid of any spaces
					$_event['am_action_data'] = $x;
					break;
			}
			if( !isset($_event['company_id']) )
				$_event['company_id'] = am_company_id();
			$event = am_update_event($_REQUEST['event']['am_event_id'], $_event);
			$event_id = $event['am_event_id'];
			fn_set_notification("N", "$controller", (empty($_REQUEST['event']['am_event_id']) ? "Created new " : "Updated ")."event '$event[am_event_name]'");
			fn_redirect("$controller.$mode.$event_id.$event[am_event_type]");
		}
		
		$event_id = $action;
		$event = array();
		if( $event_id ) {
			$event = am_get_event($event_id);
			$event_type = $event['am_event_type'];
		} else {
			$event_type = $dispatch_extra;
		}
		$action_menu = am_action_type_menu($event_type);
		$template_menu = am_template_menu($event_type);
		// 7/23/19 - deprecated in V4.10
//		$payment_info = fnX_get_payment_methods($_SESSION['auth']);
	   $params = array(
			// 11/6/19 - Change from $lang_code to CART_LANGUAGE
			'lang_code' => CART_LANGUAGE,
		);	
		$params['status'] = 'A';
		$payment_info = fn_get_payments($params);
// End deprecated solution
		
		$view = Registry::get('view');
		$view->assign('event_id', $event_id);
		if( isset($event['am_action_data']) && is_array($event['am_action_data']) )
			$event['am_action_data'] = implode(',', $event['am_action_data']);
		$view->assign('event', $event);
		$view->assign('event_type', $event_type);
		$view->assign('action_menu', $action_menu);
		$view->assign('template_menu', $template_menu);
		$view->assign('payment_info', $payment_info);
		$view->assign('order_status_menu', am_order_status_menu());
		return array(CONTROLLER_STATUS_OK);
		break;
	case 'delete_event':
		$event_id = $action;
		if( $event_id ) {
			$event = am_get_event($event_id);
			am_delete_event($event_id);
			fn_set_notification('N', "$controller", "Deleted event '$event[am_event_name]'");
			return array(CONTROLLER_STATUS_REDIRECT, "$controller.manage");
		}
		break;
	case 'run_queue':
		$items = am_get_due_items(TIME);
		if( $n = count($items) ) {
			am_run_queue($items);	// $action "can" contain 'force', or any non-empty value to force the queue to run regardless of the times.
			!defined('CONSOLE') && fn_set_notification('N', "$controller", "Processed $n queued mail items");
		} else {
			!defined('CONSOLE') && fn_set_notification('W', "$controller", "No items to send");
		}
		if( function_exists('am_event_abandoned_carts') )
			am_event_abandoned_carts();
		if( defined('CONSOLE') )
			exit;
		fn_redirect("$controller.manage");
		break;
	case 'run_queue_item':
		$am_queue_id = $action;
		$queue_status = $dispatch_extra ? $dispatch_extra : AM_QUEUE_STATUS_PENDING;
		if( $am_queue_id ) {
			$item = am_get_queued_item($am_queue_id);
			am_run_queue_item($am_queue_id, $item);
			fn_set_notification('N', "$controller", "Sent email to '$item[am_send_to]'");
		}
		fn_redirect("$controller.show_queue.$queue_status");
		break;
	case 'delete_queue_item':
		$am_queue_id = empty($_REQUEST['am_queue_id']) ? 0 : $_REQUEST['am_queue_id'];
		if( !$am_queue_id ) {
			fn_set_notification('E', "$controller.$mode", "No Queue ID", true);
		} else {
			$queued_item = am_get_queued_item($am_queue_id);
			$email = empty($queued_item['am_send_to']) ? 'unknown' : $queued_item['am_send_to'];
			am_delete_queued_item($am_queue_id);
			fn_set_notification('N', "$controller", "Deleted queue item for '$email'");
		}
		return array(CONTROLLER_STATUS_OK, "$controller.show_queue");
		break;
	case 'show_queue':
		$status = $action ? $action : AM_QUEUE_STATUS_PENDING;
		$start_time = $dispatch_extra ? $dispatch_extra : 0;
		$limit = empty($_REQUEST['limit']) ? '' : $_REQUEST['limit'];
		$items =  am_get_queued_items($status, $start_time, $end_time =0, $limit);
		$lines = array();
		foreach($items as $queue_id => $q_info) {
			$event = $q_info['am_event'];
			$lines[$queue_id]['am_send_to'] = $q_info['am_send_to'];
			$lines[$queue_id]['am_event_name'] = $event['am_event_name'];
			$lines[$queue_id]['am_action_name'] = $event['am_action_name'];
			switch($type=$event['am_event_type']) {
				case ORDER_EVENT:
					$lines[$queue_id]['object_id'] = fn_get_lang_var('order')."($q_info[am_trigger_data])";
					$lines[$queue_id]['am_template'] = preg_replace(';^order_;', '', basename($event['am_template'], '.tpl'));
					break;
				case PROFILE_EVENT:
					$lines[$queue_id]['object_id'] = fn_get_lang_var('user')."($q_info[am_trigger_data])";
					$lines[$queue_id]['am_template'] = preg_replace(';^profile_;', '', basename($event['am_template'], '.tpl'));
					break;
				case CART_EVENT:
					$lines[$queue_id]['object_id'] = fn_get_lang_var('cart')."($q_info[am_trigger_data])";
					$lines[$queue_id]['am_template'] = preg_replace(';cart_;', '', basename($event['am_template'], '.tpl'));
					break;
				// form extension
				case FORM_EVENT:
					$lines[$queue_id]['am_action_type'] = FORM_EVENT;
					$lines[$queue_id]['am_action_name'] = __("am_form_submission");
					$lines[$queue_id]['object_id'] = $event['am_action_data']['page_name'];
					$lines[$queue_id]['am_template'] = preg_replace(';form_;', '', basename($event['am_template'], '.tpl'));
					break;
			}
	
// 4/28/16: fix but for 1969 date when zero days was used
//			$lines[$queue_id]['send_date'] = date("m/d/Y", $q_info['am_next_send']);
			$lines[$queue_id]['send_date'] = date("m/d/Y", empty($q_info['am_next_send']) ? TIME : $q_info['am_next_send']);
			$lines[$queue_id]['sent_date'] = $q_info['am_sent'] ? date("m/d/Y", $q_info['am_sent']) : "not sent";
//am_dbg("$controller.$mode: event:".print_r($event,true), true);
		}
//am_dbg("$controller.$mode: lines:".print_r($lines,true), true);
//am_dbg("$controller.$mode: items:".print_r($items,true), true);
		$view = Registry::get('view');
		$view->assign('queue_status', $status);
		$view->assign('lines', $lines);
		return array(CONTROLLER_STATUS_OK);
		
	case 'preview_queued_item':
		$q_id = $action;
		if( $q_id ) {
			$q_data = am_get_queued_item($q_id);
			$event = $q_data['am_event'];
			$action_type = $event['am_action_type'];
			$body_template = am_check_template_path($q_data['lang_code'], $event['am_template'], $q_data['company_id']);
			if( empty($body_template) ) 
				die("Bad template '$event[am_template]'");

			$mail_params = am_mail_params($q_data);
			if( empty($mail_params) ) {
				die("Can't preview queued item (bad mail_params)");
			}
			$mail_params['reply-to'] = $mail_params['from'] = 'company_orders_department';
			if( $action_type == PROFILE_EVENT_ADD ) 
				$mail_params['reply-to'] = $mail_params['from'] = 'company_users_department';
//am_dbg("mail_params[data]:".print_r($mail_params['data'], true), true); exit;
			$view = Registry::get('view');
			foreach($mail_params['data'] as $k => $v)
				$view->assign($k, $v);
			$company_data = fn_get_company_placement_info($mail_params['company_id']);
			$view->assign('company_data', $company_data);
			$body = $view->displayMail($body_template, false, AREA, $mail_params['company_id'], $q_data['lang_code']);
			echo $body;
			echo "<br/><br/>";
			echo "<a href=\"?dispatch=auto_mail.show_queue.P\">Return to Auto Mail queue</a>";
			exit; 
		}
		die("No queue_id");
	case 'check_table':
		am_check_table_fields();
		fn_redirect("auto_mail.manage");
		break;
	case 'init_events':
		am_init_events();
		fn_set_notification("N", __("notice"), "Reset events data", 'K');
		fn_redirect("$controller.manage");
		break;
	case 'show_queue_item':	// Test
		$queue_id = $action;
		if( empty($queue_id) )
			die("No queue_id specified as \$action");
		$queue_item = am_get_queued_item($queue_id);
		die(sprintf("<pre>Item:%s</pre>", print_r($queued_item,true)));
		
	case 'preview_template':	// tests.preview_template.order_review_products&order_id=123
		fn_define('am_template_testing', true);
		$template_name = empty($action) ? "order_review_products" : $action;
		$template_name .= ".tpl";
		$queue_id = empty($_REQUEST['queue_id']) ? 0 : $_REQUEST['queue_id'];
		$event_type = ORDER_EVENT;
		if( empty($queue_id) ) {	// get one dynamically
			switch(substr($template_name, 0, 4) ) {
				case 'orde':
					$event_type = ORDER_EVENT;
					break;
				case 'prof':
					$event_type = PROFILE_EVENT;
					break;
				case 'cart':
					$event_type = CART_EVENT;
					break;
				case 'form':
					$event_type = FORM_EVENT;
					break;
			}
			$queue_id = db_get_field("SELECT am_queue_id from ?:auto_mail_queue 
									 WHERE am_event_type=?s 
									 ORDER BY am_queue_id DESC LIMIT 1", $event_type);
			if( empty($queue_id) ) {
				fn_set_notification("E", __("error"), str_replace('[TYPE]', $event_type, __("am_no_queued_event")), 'K');
				fn_redirect("auto_mail.manage");
			}
		}
																							
		$q_data = am_get_queued_item($queue_id);
		if( empty($q_data) )
			die("Bad queue_id=$queue_id");
		$event = $q_data['am_event'];
			
		$order_id = empty($q_data['am_object_id']) ? 0 : $q_data['am_object_id'];
		
		$mail_params = am_mail_params($q_data);
		if( empty($mail_params) ) {
			die("Can't preview queued item");
		}
		$mail_params['reply-to'] = $mail_params['from'] = 'company_orders_department';
		$view = Registry::get('view');
		foreach($mail_params['data'] as $k => $v)
			$view->assign($k, $v);
		$company_data = fn_get_company_placement_info($mail_params['company_id']);
		$view->assign('company_data', $company_data);
		$body = $view->displayMail($mail_params['tpl'], false, AREA, $mail_params['company_id'], 'en');
		echo $body;
		echo "<br/><br/>";
		echo "<a href=\"?dispatch=auto_mail.manage\">Return to Auto Mail manage page</a>";
		exit; 
		break;
		
	case 'change_templates':
		$from = empty($_REQUEST['from']) ? '' : $_REQUEST['from'];
		$to = empty($_REQUEST['to']) ? '' : $_REQUEST['to'];
		$queue_status = empty($_REQUEST['status']) ? 'P' : $_REQUEST['status'];
		if( empty($from) || empty($to) || empty($queue_status) ) {
			fn_set_notification("E", __("error"), "No From, To or Status set", 'K');
			fn_redirect("auto_mail.manage");
		}
		$cnt = 0;
		foreach( db_get_array("SELECT am_queue_id, am_event FROM ?:auto_mail_queue WHERE am_queue_status=?s", $queue_status) as $qd) {
			$event = $qd['am_event'] ? unserialize($qd['am_event']) : '';
			if( $event && $event['am_template'] == $from ) {
				$event['am_template'] = $to;
				$cnt++;
				$event = serialize($event);
				db_query("UPDATE ?:auto_mail_queue set am_event=?s WHERE am_queue_id=?i",
						 	$event, $qd['am_queue_id']);
			}
		}
		$msg = $cnt 
				? "Updated '$cnt' templates from '$from' to '$to' for queue status '$queue_status'"
				: "No matching templates found for '$from' for queue status '$queue_status'";
		fn_set_notification($cnt?"N":"W", $cnt?__("notice"):__("warning"), $msg, 'K');
		fn_redirect("auto_mail.show_queue.$queue_status");
		break;
		
	case 'show_unsubscribed':
		$ar = db_get_fields("SELECT email FROM ?:auto_mail_subscribers WHERE status='N'");
		foreach($ar as $em) {
			$uid = db_get_field("SELECT user_id FROM ?:users WHERE email=?s", $em);
			if( $uid )
				printf('<a href="%s">%s</a><br/>', fn_url("profiles.update&amp;user_id=$uid&amp;selected_section=addons"), $em);
			else
				printf("%s<br/>", $em);
		}
		if( empty($ar) )
			printf("<h3>%s</h3>", __("am_noone_unsubscribed"));
		exit;

		
	// Tests and temporary
	case 'test':
	case 'tests':
		include(dirname(__FILE__)."/tests.php");
		die("test.$action: shouldn't get here!");
	default:
		if( file_exists($f = ez_fix_path($dir_addons."ez_common/lib/std_addon_controller.php")) ) {
			include($f);
		}
		break;
}

$return_url = !empty($return_url)
				? $return_url 
				: (!empty($_REQUEST['redirect_url'])
					? $_REQUEST['redirect_url']
					: (!empty($_REQUEST['return_url'])
						? $_REQUEST['return_url']
						: $index_script
						)
				);

return array(CONTROLLER_STATUS_OK, $return_url);
?>