<?php
/*****************************
* Copyright 2021 1st Source IT, LLC
* All rights reserved.
* Permission granted for use as
* long as this copyright notice, associated text and
* links remain in tact.
* Licensed for a single domain and a single instance of EZ-cart.
* Additional licenses can be purchased for additonal sites.
*
* http://www.ez-ms.com
* http://www.ez-om.com
* http://www.1sit.com*
*
* End copyright notification
*/

/* Every addon must have this to bootstrap the ez_common addon
 */
if( !defined('BOOTSTRAP') ) die('Access denied');

use Tygh\Registry;
use Tygh\Mailer;

if( file_exists($f=ez_fix_path(Registry::get('config.dir.addons')."ez_common/lib/upgrade.php")) ) {
	require_once($f);
} else {	// Need to force update of the tools
	require_once(ez_fix_path(Registry::get('config.dir.addons')."$controller/lib/upgrade/upgrade.php"));		// copy of common version of tools - used to bootstrap the ez_common addon
}

switch($mode) {
	case 'order_reviews':
		$am_queue_id = empty($_REQUEST['r_key']) ? 0 : $_REQUEST['r_key'];
		
		if( !(Registry::get('addons.discussion.status') == 'A') || !$am_queue_id ) {
			fn_set_notification('E', __("am_mail_reviews"),$s= __("am_review_not_available"), 'K');
			am_log("$controller.$mode: Discussion not active");
			return array(CONTROLLER_STATUS_DENIED, fn_url("") );
		}
		
		$show_testimonial = am_show_testimonial() == 'Y';
		$testimonial = array();
		
		$view = Tygh::$app['view'];
		
		// Common to both GET and POST
		
		// Check validity
		if( !($am_q = am_get_queued_item($am_queue_id)) 
			|| ($am_q['am_queue_status'] != AM_QUEUE_STATUS_SENT && !defined('am_template_testing')) ) {
			am_log("$controller.$mode: ".($s="Bad queue status '$am_queue_id/$am_q[am_queue_status]'") );
			fn_set_notification('E', __("error"), $s, 'K');
			return array(CONTROLLER_STATUS_DENIED, fn_url(""));
		}
		$company_id = $am_q['company_id'];
		$user_id = db_get_field("SELECT user_id FROM ?:orders WHERE order_id=?i", $am_q['am_object_id']);
		$name = $user_id ? db_get_field("SELECT concat(firstname, ' ', lastname) as name FROM ?:users WHERE user_id=?i", $user_id) : "Anonymous";
		if( empty($name) )
			$name = "Anonymous";
		
		$discussion_settings = Registry::get('addons.discussion');
		
		if( $_SERVER['REQUEST_METHOD'] == 'POST' ) {
			/* Required params
			 *	- r_key - am_queue_id
			 *	- products or testimonial
			 *	- order_id
			 */
			$discussion_object_types = fn_get_discussion_objects();
			$products = $_REQUEST['products'];
			$testimonial_active = !empty($_REQUEST['testimonial']['active']) && $_REQUEST['testimonial']['active'] == 'Y';
			if( $show_testimonial && $testimonial_active ) {
				$testimonial = $_REQUEST['testimonial'];
			}
				
			
				
			$ip_data = fn_get_ip();
			$ip = empty($ip_data['host']) ? 0 : $ip_data['host'];
			
			// Update the testimonial reviews
			$thread_id = 0;
			$testimonial_discussion = fn_get_discussion(0, 'E');
			if( $testimonial_discussion )
				$thread_id = empty($testimonial_discussion['thread_id']) ? 0 : $testimonial_discussion['thread_id'];
			if( $testimonial && $thread_id ) {
				$object = am_discussion_object_by_thread($thread_id);
				$object_data = fn_get_discussion_object_data($object['object_id'], $object['object_type']);
				$object_name =  $discussion_object_types[$object['object_type']];
				
				if (AREA != 'A' && !empty($discussion_settings[$object_name . '_post_approval'])) {
					if ($discussion_settings[$object_name . '_post_approval'] == 'any' 
					|| ($discussion_settings[$object_name . '_post_approval'] == 'anonymous' 
						&& empty($user_id))) {
						
						fn_set_notification('W', __('text_thank_you_for_post'), __('text_post_pended'));
						$testimonial['status'] = 'D';
					}
				}
				$testimonial['thread_id'] = $thread_id;
				$testimonial['timestamp'] = TIME;
				$testimonial['user_id'] = $user_id;
				$testimonial['ip_address'] = $ip;
				$testimonial['name'] = $name;
				$testimonial['rating_value'] = $testimonial[0]['rating'];
				unset($testimonial[0]);
				$testimonial['post_id'] = db_query("INSERT INTO ?:discussion_posts ?e", $testimonial);
				db_query("REPLACE INTO ?:discussion_messages ?e", $testimonial);
				db_query("REPLACE INTO ?:discussion_rating ?e", $testimonial);				
			} elseif( $testimonial) {
				fn_set_notification('E', __("error"), __("am_testimonial_reviews_not_enabled"));
				am_log("$controller.$mode: testimonial reviews not enabled.");
			}
				
			
			// Now process each product
			$reviewed_products = 0;
			
			foreach($products as $product_id => $post_data) {
				if( $post_data['active'] != 'Y' ) {
					unset($products[$product_id]);
					continue;
				}
				$thread_id = am_product_thread_id($product_id, $company_id);
				$object = am_discussion_object_by_thread($thread_id);
				if( empty($object) ) {
					fn_set_notification('E', __('error'), $s=str_replace('[%product%]', $post_data['product_code'], __('am_reviews_not_enabled')), 'K');
					am_log("$controller.$mode: reviews not enabled for product: $s");
					continue;
				}
				$object_name =  $discussion_object_types[$object['object_type']];
				if( $object_name != 'product' )
					continue;
				$object_data = fn_get_discussion_object_data($object['object_id'], $object['object_type']);
				// Check if post needs to be approved
				if (AREA != 'A' && !empty($discussion_settings[$object_name . '_post_approval'])) {
					if ($discussion_settings[$object_name . '_post_approval'] == 'any' 
					|| ($discussion_settings[$object_name . '_post_approval'] == 'anonymous' 
						&& empty($user_id))) {
						
						fn_set_notification('W', __('text_thank_you_for_post'), __('text_post_pended'));
						$post_data['status'] = 'D';
					}
				}
				$post_data['thread_id'] = $thread_id;
				$post_data['timestamp'] = TIME;
				$post_data['user_id'] = $user_id;
				$post_data['name'] = $name;
				$post_data['ip_address'] = $ip;
				$post_data['rating_value'] = $post_data['rating'];
				$post_data['post_id'] = db_query("INSERT INTO ?:discussion_posts ?e", $post_data);
				db_query("REPLACE INTO ?:discussion_messages ?e", $post_data);
				db_query("REPLACE INTO ?:discussion_rating ?e", $post_data);
				$reviewed_products++;
				
				// Send notifications
				// Admin
				if (!empty($discussion_settings[$object_name . '_notification_email'])) {
					$lang_code = Registry::get('settings.Appearance.backend_default_language');
					$url = "discussion_manager.manage?object_type=$object[object_type]&post_id=$post_data[post_id]";

					$data = array(
						'url' => fn_url($url, 'A', 'http', null, true),
						'object_data' => $object_data,
						'post_data' => $post_data,
						'object_name' => $object_name,
						'subject' => __('discussion_title_' . $discussion_object_types[$object['object_type']], '', $lang_code) . ' - ' . __($discussion_object_types[$object['object_type']], '', $lang_code)
						);
					$to = empty($discussion_settings[$object_name . '_notification_email']) 
							? 'site_administrator'
							: $discussion_settings[$object_name . '_notification_email'];
					am_send_notification($to,
										$from='company_site_administrator',
										$data,
										$tpl='addons/discussion/notification.tpl',
										$company_id,
										$lang_code,
										$area = 'A');											
				}	// send to admin

                //Send to vendor
				if(!empty($company_id) && !empty($discussion_settings[$object_name . '_notify_vendor']) && $discussion_settings[$object_name . '_notify_vendor'] == 'Y') {
					$lang_code = fn_get_company_language($company_id);
                    $url = ($object_name == 'company' ? 'companies' : $object_name) . "s.update?$object_name" . "_id=$object[object_id]&selected_section=discussion";
					$data = array(
                            'url' => fn_url($url, 'V', 'http', null, true),
                            'object_data' => $object_data,
                            'post_data' => $post_data,
                            'object_name' => $object_name,
                            'subject' => __('discussion_title_' . $discussion_object_types[$object['object_type']], '', $lang_code) . ' - ' . __($discussion_object_types[$object['object_type']], '', $lang_code)
                        );
								  
					am_send_notification($to='company_site_administrator',
										$from='default_company_site_administrator',
										$data,
										$tpl='addons/discussion/notification.tpl',
										$company_id,
										$lang_code,
										$area = 'A');											
				}	// send to vendor
			}	// foreach
			if( $reviewed_products )
				fn_set_notification('N', __("am_thank_you"), __("am_for_reviewing"), 'K');
			fn_redirect("");
		} else {		// GET
			/* Expects following params
			 *	- (optional) default_rating (1-5)
			 *	- r_key: auto_mail queue id
			 *	- (optional) product_ids - comma separated list of product_ids
			 */
			 
			if( $user_id && (empty($_SESSION['auth']['user_id']) || $_SESSION['auth']['user_id'] != $user_id) ) {
				// Not logged in or user_id missmatch with order
				fn_user_logout($_SESSION['auth']);
				$params = array('product_ids', 'r_key', 'default_rating');
				$qs = array();
				foreach($params as $p) {
					if(isset($_REQUEST[$p]) )
						$qs[] = $p ."=" . $_REQUEST[$p];
				}
				$q_str = implode('&amp;', $qs);
				$return_url = urlencode(sprintf("%s.%s&amp;%s", $controller, $mode, $q_str));
				fn_set_notification('W', __("warning"), __("am_must_login"), 'K');
				fn_redirect("auth.login_form&amp;return_url=$return_url");
			}
			
			$product_ids = explode(',', $_REQUEST['product_ids']);
			if( empty($product_ids[0]) )
				$product_ids = array();
			$products = array();
			if( !empty($product_ids) ) {
				$params = array('pname'=>'Y', 'pid'=>$product_ids);
				$products = current(fn_get_products($params));
				foreach($products as $product_id => &$p_info) {
					$pair_data = fn_get_image_pairs($product_id, 'product', 'M', true, true, CART_LANGUAGE);
					if( $pair_data )
						$p_info['main_pair'] = $pair_data;
				}
			}
		}
		$view->assign('show_testimonial', $show_testimonial ? 'Y' : 'N');
		$view->assign('products', $products);
		$dflt_rating = Registry::ifget("addons.auto_mail.default_rating", "4");
		$view->assign('default_rating', empty($_REQUEST['default_rating']) 
										? $dflt_rating
										: $_REQUEST['default_rating']);
		$view->assign('r_key', $am_queue_id);
		return array(CONTROLLER_STATUS_OK);
		break;

	case 'review':
//am_log("$controller/$mode: REQUEST:".print_r($_REQUEST,true) );
		if( Registry::get('addons.discussion.status') == 'A' && !empty($_REQUEST['post_data']) ) {
//am_dbg("<pre>post_data:".print_r($_REQUEST,true), true);
			if( !empty($_REQUEST['preview']) ) {
				fn_set_notification("E", "Auto-Mail", "Preview review templates cannot be submitted.", 'K');
				fn_redirect("auto_mail.show_queue.P");
			}
			$pd = $_REQUEST['post_data'];
			$am_queue_id = empty($_REQUEST['r_key']) ? 0 : $_REQUEST['r_key'];
			// Check validity
			if( !($am_q = am_get_queued_item($am_queue_id)) 
				|| $am_q['am_object_id'] != $_REQUEST['order_id']
				|| ($am_q['am_queue_status'] != AM_QUEUE_STATUS_SENT && !defined('am_template_testing')) ) {
				die("Bad queue item");
			}
			
			$discussion_settings = Registry::get('addons.discussion');
			$discussion_object_types = fn_get_discussion_objects();
			// Now process each product
			$reviewed_products = 0;
			foreach($pd['products'] as $product_id => $post_data) {
				if( !empty($post_data['thread_id']) 
					&& (!empty($post_data['message']) || $post_data['rating'] > 0) ) {
					$object = am_discussion_object_by_thread($post_data['thread_id']);
					if( empty($object) ) {
						fn_set_notification('E', __('error'), $s=str_replace('[%product%]', $post_data['product_code'], __('am_reviews_not_enabled')), 'K');
						am_log("$controller.$mode: $post_data[name]: $s");
						continue;
					}
					$object_name =  $discussion_object_types[$object['object_type']];
					if( $object_name != 'product' )
						continue;
					$object_data = fn_get_discussion_object_data($object['object_id'], $object['object_type']);
					$post_data['status'] = 'A';
					// Check if post needs to be approved
					if (AREA != 'A' && !empty($discussion_settings[$object_name . '_post_approval'])) {
						if ($discussion_settings[$object_name . '_post_approval'] == 'any' || ($discussion_settings[$object_name . '_post_approval'] == 'anonymous' && empty($pd['user_id']))) {
							
							fn_set_notification('W', __('text_thank_you_for_post'), __('text_post_pended'));
							$post_data['status'] = 'D';
						}
					}
					$post_data['timestamp'] = TIME;
					$post_data['user_id'] = (integer)(empty($pd['user_id']) 
														? (empty($_REQUEST['user_id']) ? 0 : $_REQUEST['user_id'])
														: $pd['user_id']);
					$post_data['post_id'] = db_query("INSERT INTO ?:discussion_posts ?e", $post_data);
//die("<pre>post_data:".print_r($post_data,true));
					db_query("REPLACE INTO ?:discussion_messages ?e", $post_data);
					db_query("REPLACE INTO ?:discussion_rating ?e", $post_data);
					$reviewed_products++;
				} else {
					continue;
				}
				
				if (!empty($discussion_settings[$object_name . '_notification_email']) 
						|| (!empty($discussion_settings[$object_name . '_notify_vendor']) 
					&& $discussion_settings[$object_name . '_notify_vendor'] == 'Y')) {		
					$company_id = 0;
					if (fn_allowed_for('MULTIVENDOR')) {
						$company_id = db_get_field("SELECT company_id FROM ?:products WHERE product_id = ?i", $object['object_id']);
					}
                }

				$url = "discussion_manager.manage?object_type=$object[object_type]&post_id=$post_data[post_id]";

				if (!empty($discussion_settings[$object_name . '_notification_email'])) {
					Mailer::sendMail(array(
                        'to' => $discussion_settings[$object_name . '_notification_email'],
                        'from' => 'company_site_administrator',
                        'data' => array(
                            'url' => fn_url($url, 'A', 'http', null, true),
                            'object_data' => $object_data,
                            'post_data' => $post_data,
                            'object_name' => $object_name,
                            'subject' => __('discussion_title_' . $discussion_object_types[$object['object_type']], '', Registry::get('settings.Appearance.backend_default_language')) . ' - ' . __($discussion_object_types[$object['object_type']], '', Registry::get('settings.Appearance.backend_default_language')),
                        ),
                        'tpl' => 'addons/discussion/notification.tpl',
                        'company_id' => $company_id,
                    ), 'A', Registry::get('settings.Appearance.backend_default_language'));
				}	// send to admin

                //Send to vendor
				if (!empty($company_id) && !empty($discussion_settings[$object_name . '_notify_vendor']) && $discussion_settings[$object_name . '_notify_vendor'] == 'Y') {

                    $url = ($object_name == 'company' ? 'companie' : $object_name) . "s.update?$object_name" . "_id=$object[object_id]&selected_section=discussion";
                    Mailer::sendMail(array(
                        'to' => 'company_site_administrator',
                        'from' => 'default_company_site_administrator',
                        'data' => array(
                            'url' => fn_url($url, 'V', 'http', null, true),
                            'object_data' => $object_data,
                            'post_data' => $post_data,
                            'object_name' => $object_name,
                            'subject' => __('discussion_title_' . $discussion_object_types[$object['object_type']], '', fn_get_company_language($company_id)) . ' - ' . __($discussion_object_types[$object['object_type']], '', fn_get_company_language($company_id)),
                        ),
                        'tpl' => 'addons/discussion/notification.tpl',
                        'company_id' => $company_id,
                    ), 'A', fn_get_company_language($company_id));
				}	// send to vendor
			}	// foreach
			if( $reviewed_products )
				fn_set_notification('N', __("am_thank_you"), __("am_for_reviewing"), 'K');
		} else {
			fn_set_notification('E', __("am_mail_reviews"),$s= __("am_review_not_available"), 'K');
			am_log("$controller.$mode: Discussion not active");
		}
		fn_redirect("");
		
	case 'version':
		$ez_common = 'ez_common';
		addon_init($ez_common);
		$ez_version = addon_current_version($ez_common);
		if( empty($ez_version) )
			$ez_version = "not installed or missconfigured";
		addon_init($controller);
		$version = addon_current_version($controller);
		$msg = PRODUCT_NAME.": ".PRODUCT_VERSION." ".PRODUCT_EDITION."<br/>EZ Common: $ez_version<br/>$controller: $version";
		fn_set_notification('N', "$controller.$mode", $msg, true);
		fn_redirect(Registry::get('config.customer_index'));
		break;
	case 'unsubscribe':
		// Some validation
		// Required for get
		$am_queue_id = !empty($_REQUEST['r_key']) ? $_REQUEST['r_key'] : 0;
		if( empty($am_queue_id) )
			die("Bad mail queue");
			
		if( !($am_q = am_get_queued_item($am_queue_id)) 
			|| ($am_q['am_queue_status'] != AM_QUEUE_STATUS_SENT && !defined('am_template_testing')) ) {
			am_log("$controller.$mode: ".($s="Bad queue status '$am_q[am_queue_status]'") );
			fn_set_notification('E', __("error"), $s, 'K');
			return array(CONTROLLER_STATUS_DENIED, fn_url(""));
		}
		
		if( $_SERVER['REQUEST_METHOD'] == 'POST' ) {
			$status = $_REQUEST['status'];
			$email = trim($_REQUEST['email']);
			$lang_code = $_REQUEST['lang_code'];
			$u_subscribe = am_get_user_subscriber($email);
			if( $u_subscribe != $status )
				am_set_user_subscriber($email, $status);
			am_put_subscriber($email, $lang_code, $status);
			if( $status == 'N' )
				fn_set_notification("N", __("notice"), __("am_unsubscribed"), 'K');
			fn_redirect(Registry::get('config.customer_index'));
		}
		
		$email = trim($am_q['am_send_to']);
		$lang_code = $am_q['lang_code'];
		
		$view = Tygh::$app['view'];
		$view->assign('email', $email);
		$view->assign('lang_code', $lang_code);
		$view->assign('r_key', $am_queue_id);
		$view->assign('subscriber_info', am_get_subscriber($email, $lang_code));
		$view->assign('current_status', am_subscriber_status($email, $lang_code));
		return array(CONTROLLER_STATUS_OK);	
		
	case 'test':
	case 'tests':
		if( file_exists($f=__DIR__."/tests.php") )
			include($f);
		die("No such mode '$mode'");

}
return array(CONTROLLER_STATUS_NO_PAGE);
?>