<?php

/*****************************
* Copyright 2021 1st Source IT, LLC
* All rights reserved.
* Permission granted for use as
* long as this copyright notice, associated text and
* links remain in tact.
* Licensed for a single domain and a single instance of EZ-cart.
* Additional licenses can be purchased for additonal sites.
*
* http://www.ez-ms.com
* http://www.ez-om.com
* http://www.1sit.com*
*
* End copyright notification
*/
if( !defined('BOOTSTRAP') ) die('Access denied');

use Tygh\Registry;
function am_dispatch_order_event($order_id, &$order, $new_status) {
	// 11/27/16: Add pre hook
	$allow = true;
	fn_set_hook('am_dispatch_order_event_pre', $order_id, $order, $new_status, $allow);
	if( !$allow )
		return array();
	
	$company_id = isset($order['company_id']) ? $order['company_id'] : am_company_id();
	am_log(__FUNCTION__.": order_id=$order_id, company_id=$company_id, new_status=$new_status", logDetail);
	$queue_data = array();
	$default_queue_data = array('am_event'=>array(), 
								'am_event_type'=>ORDER_EVENT, 
								'am_object_id'=>$order_id, 
								'am_queue_status'=>AM_QUEUE_STATUS_PENDING, 
								'am_next_send'=>0, 
// 1/20/16: Fix bug related to queued lang_code
								'lang_code'=>empty($order['lang_code']) ? CART_LANGUAGE : $order['lang_code'], 
								'am_send_to'=>$order['email'], 
								'company_id' => $order['company_id']
								);
	$action_cnt = 0;
	foreach(am_get_events(ORDER_EVENT, 'A', $company_id, true) as $event_id => $event_info) {

		$compare_status = empty($event_info['am_event_options']['order_status']) 
							? Registry::get('addons.auto_mail.am_status') 
							: $event_info['am_event_options']['order_status'];
		if( ($new_status != $compare_status) ) {
			am_log(__FUNCTION__.": NO STATUS MATCH: name=$event_info[am_event_name]($event_id), new_status=$new_status, compare_status=$compare_status", logDetail );
			continue;
		}
		$action_hit = false;
//am_dbg("\t: new_status=$new_status, processing event '$event_id', am_action_type=$event_info[am_action_type]" );
		switch($event_info['am_action_type']) {
			case ORDER_EVENT_PLACED:		// Always true
				$idx = $action_cnt++;
				$queue_data[$idx] = $default_queue_data;
				$queue_data[$idx]['am_event'] = $event_info;
				$queue_data[$idx]['am_next_send'] = am_next_send($event_info['am_days']);	//(SECONDS_IN_DAY * $event_info['am_days']) + TIME;
				$queue_data[$idx]['am_trigger_data'] = $order_id;
				am_log(__FUNCTION__.": Hit on place_order event ($event_info[am_action_type])", logDetail);
				$action_hit = true;
				break;
			case ORDER_EVENT_PRODUCTS_PURCHASED:
				// 10/28/13 - If multiple events are defined, then $p = array() could set
				// All the items empty except foor the first event.
//				$p = array();
				// 7/23/15: Changed from 'items' to 'products'  V4 issue.
				if( !empty($order['products']) )
					$p =& $order['products'];
				else
					$p = array();
				$p_ar =& $event_info['am_action_data'];
				foreach($p as $idx => $p_info) {
					$pcode = am_get_product_code($p_info['product_id']);
					if( in_array($pcode, $event_info['am_action_data']) ) {
						$idx = $action_cnt++;
						$queue_data[$idx] = $default_queue_data;
						if(!empty($event_info['am_event_options']['days_x_quantity']) && ($event_info['am_event_options']['days_x_quantity'] == 'Y') )
							$event_info['am_days'] *= $p_info['amount'];
						$queue_data[$idx]['am_event'] = $event_info;
						$queue_data[$idx]['am_next_send'] = am_next_send($event_info['am_days']);
						$queue_data[$idx]['am_trigger_data'] = $pcode;
						am_log(__FUNCTION__.": Hit on Products purchased event (pcode='$pcode') for order_id=$order_id", logDetail);
						$action_hit = true;
						break;
					}
				}
				if( !$action_hit )
					am_log(__FUNCTION__.": NO Hit on product codes for order_id=$order_id", logDetail);
				break;
			case ORDER_EVENT_PAYMENT_TYPE:
				$o = fn_get_order_info($order_id);
				if( !empty($o['payment_method']['payment_id']) && ($o['payment_method']['payment_id'] == $event_info['am_action_data']) ) {
					$idx = $action_cnt++;
					$queue_data[$idx] = $default_queue_data;
					$queue_data[$idx]['am_event'] = $event_info;
					$queue_data[$idx]['am_next_send'] = am_next_send($event_info['am_days']);
					$queue_data[$idx]['am_trigger_data'] = $o['payment_method']['payment_id'];
					am_log(__FUNCTION__.": Hit on Payment type event ($event_info[am_action_type]) for order_id='$order_id'", logDetail);
					$action_hit = true;
				}
				if( !$action_hit )
					am_log(__FUNCTION__.": NO Hit on payment type ($event_info[am_action_data]) for order_id=$order_id", logDetail);
				break;
			case ORDER_EVENT_AMOUNT_GREATER:
				if( $order['subtotal'] > $event_info['am_action_data'] ) {
					$idx = $action_cnt++;
					$queue_data[$idx] = $default_queue_data;
					$queue_data[$idx]['am_event'] = $event_info;
					$queue_data[$idx]['am_next_send'] = am_next_send($event_info['am_days']);
					$queue_data[$idx]['am_trigger_data'] = $order['subtotal'];
					am_log(__FUNCTION__.": Hit on Amount greater event (subtotal=$order[subtototal], amount=$event_info[am_action_type]) for order_id='$order_id'", logDetail);
					$action_hit = true;
				}
				if( !$action_hit )
					am_log(__FUNCTION__.": NO Hit on Amount greater ($event_info[am_action_data]) for order_id=$order_id. subtotal is '$order[subtotal]'", logDetail);
				break;
			case ORDER_EVENT_SHIP_METHOD:
				$ship_ids = explode(',', $order['shipping_ids']);
				if( in_array($event_info['am_action_data'], $ship_ids) ) {
					$idx = $action_cnt++;
					$queue_data[$idx] = $default_queue_data;
					$queue_data[$idx]['am_event'] = $event_info;
					$queue_data[$idx]['am_next_send'] = am_next_send($event_info['am_days']);
					$queue_data[$idx]['am_trigger_data'] = $event_info['am_action_data'];
					am_log(__FUNCTION__.": Hit on Shipping id event (shipping_id=$event_info[am_action_data] for order_id='$order_id'", logDetail);
					$action_hit = true;
				}
				if( !$action_hit )
					am_log(__FUNCTION__.": NO Hit on Shipping_id $event_info[am_action_data]", logDetail);
				break;
					
		}
		if( $action_hit && !empty($event_info['am_event_options']['stop_other_actions']) && ($event_info['am_event_options']['stop_other_actions'] == 'Y') )
			break;
	}	// foreach event
	foreach($queue_data as $idx => $_queue_data)		// Event was found
		$queue_data[$idx] = am_update_queue(0, $_queue_data);
	return $queue_data;
}

function am_dispatch_profile_event($user_id, $user_data, $types=array(AM_USER_TYPE_ALL)) {
	$company_id = am_company_id();
	$queue_data = array();
	$default_queue_data = array('am_event'=>array(), 
								'am_event_type'=>PROFILE_EVENT, 
								'am_object_id'=>$user_id, 
								'am_queue_status'=>AM_QUEUE_STATUS_PENDING, 
								'am_next_send'=>0, 
// 1/20/16: Fix bug related to queued lang_code
								'lang_code'=>empty($user_data['lang_code']) ? CART_LANGUAGE : $user_data['lang_code'], 
								'am_send_to'=>$user_data['email'], 
								'company_id' => $user_data['company_id'] 
								);
	$action_cnt = 0;
	$u = fn_get_user_info($user_id);
	foreach(am_get_events(PROFILE_EVENT, 'A', $company_id, true) as $event_id => $event_info) {
		
		// If "all" is not set
		if( !in_array(AM_USER_TYPE_ALL, $types) ) {
			// Check that this one is active
			$event_user_types = empty($event_info['am_event_options']['user_types'])
							? array(AM_USER_TYPE_ALL)
							: $event_info['am_event_options']['user_types'];
			if( !in_array($u['user_type'], $event_user_types) ) {	// Not for this user_type
				if( !in_array(AM_USER_TYPE_ALL, $event_user_types) )	// And all not set for this event
					continue;
			}
		}
		// Event should be valid for this type.
		$action_hit = false;
		switch($event_info['am_action_type']) {
			case PROFILE_EVENT_ADD:
				$idx = $action_cnt++;
				$queue_data[$idx] = $default_queue_data;
				$queue_data[$idx]['am_event'] = $event_info;
				$queue_data[$idx]['am_next_send'] = (SECONDS_IN_DAY * $event_info['am_days']) + TIME;
				$queue_data[$idx]['am_trigger_data'] = $user_id;
				am_log(__FUNCTION__.": Hit on ADD event ($user_data[email]) for user_id='$user_id'", logDetail);
				$action_hit = true;
				break;
		}
		if( $action_hit && !empty($event_info['am_event_options']['stop_other_rules']) && ($event_info['am_event_options']['stop_other_rules'] == 'Y') )
			break;
	}
	foreach($queue_data as $idx => $_queue_data)
		$queue_data[$idx] = am_update_queue(0, $_queue_data);
	return $queue_data;
}

function am_dispatch_form_event($page_id, $form_data) {
	$log_level = logLog;
	$event_type = FORM_EVENT;
	$company_id = am_company_id();
	$queue_data = array();
	$events = am_get_events($event_type, 'A', $company_id, true);
	foreach($events as $event_id => $ev_data) {
		if( $ev_data['am_action_data'] != $page_id ) 
			unset($events[$event_id]);
	}
	if( empty($events) ) {
		am_log(__FUNCTION__.": No events for page_id=$page_id", empty($log_level) ? logDetail : logLog);
		return;
	}
	// $events now contains events with object_id as page_id
			
	$qd = array('am_event'=>array(), 'am_event_type'=>$event_type,
								'am_action_type' => $event_type,
								'am_action_name' => __("am_form_submission"),
								'am_action_id'=>$page_id,
								'am_queue_status'=>AM_QUEUE_STATUS_PENDING, 
								'am_next_send'=>0, 'lang_code'=>CART_LANGUAGE, 
								'am_trigger_data' => '',
								'am_send_to'=>$form_data['email'], 'company_id' => $company_id );
	$cnt = 0;
	foreach($events as $event_id => $ev_data) {
		$ev_data['am_action_data'] = $form_data;
		$qd['am_event'] = $ev_data;
		// 5/5/16: Fixed am_next send to be a TIME rather than days.
		$qd['am_next_send'] = (SECONDS_IN_DAY * $ev_data['am_days']) + TIME;
		am_update_queue(0, $qd);
		$cnt++;
	}
	return $cnt;
	
}

function am_dispatch_cart_event($cart) {
	$company_id = am_company_id();
	$queue_data = array();
	$user_id = $cart['user_id'];
	$timestamp = $cart['timestamp'];
	$queue_data = array();
	$default_queue_data = array('am_event'=>array(), 
								'am_event_type'=>CART_EVENT, 
								'am_object_id'=>0, 
								'am_queue_status'=>AM_QUEUE_STATUS_PENDING, 
								'am_next_send'=>0, 
// 1/20/16: Fix bug related to queued lang_code
								'lang_code'=>empty($cart['lang_code']) ? CART_LANGUAGE : $cart['lang_code'], 
								'am_send_to'=>$cart['user_data']['email'], 
								'company_id' => !isset($cart['company_id']) ? $company_id : $cart['company_id']
								);
	$action_cnt = 0;
	$u = $user_id ? fn_get_user_info($user_id) : array();
	$am_age = '';
	
	// Note am_get_events for CART_EVENT returnes in order of biggest 'abandon_age' first
	foreach(am_get_events(CART_EVENT, 'A', $company_id, true) as $event_id => $event_info) {
		$am_age = $event_info['am_event_options']['abandon_age'];
		$age_seconds = am_abandon_age_ts($am_age);
		if( ($timestamp > (TIME - $age_seconds) )
		||	(in_array($am_age, $cart['am_ages'])) )
			continue;	// Not yet or already done for this event
		
		// Event should be valid for this type.
		$action_hit = false;
		switch($event_info['am_action_type']) {
			case CART_EVENT_ABANDONED:
				$idx = $action_cnt++;
				$queue_data[$idx] = $default_queue_data;
				$queue_data[$idx]['am_event'] = $event_info;
				$queue_data[$idx]['am_event']['am_action_data'] = $cart['session_id'];
				$queue_data[$idx]['am_next_send'] = 0;	// Always immediate
				$queue_data[$idx]['am_trigger_data'] = $am_age;
				am_log(sprintf("%s: Hit on ADD event (%s) for user_id='%d'", __FUNCTION__, $cart['user_data']['email'], $user_id), logDetail);
				$action_hit = true;
				break;
		}
		if( $action_hit && !empty($event_info['am_event_options']['stop_other_rules']) && ($event_info['am_event_options']['stop_other_rules'] == 'Y') )
			break;
	}
	foreach($queue_data as $idx => $_queue_data) {
		$queue_data[$idx] = am_update_queue(0, $_queue_data);
		if( $queue_data[$idx] ) {
			am_update_ages($_queue_data['am_trigger_data'], $cart['session_id']);
			// For this action, we run the queue immediately for each event triggered
			am_run_queue_item($queue_data[$idx]['am_queue_id'], $queue_data[$idx]);
		}
	}	
	if( $queue_data && ($cnt=count($queue_data)) ) {
		$msg = str_replace('[COUNT]', $cnt, __("am_cart_sent_count"));
		if(!defined('CONSOLE') ) 
			fn_set_notification("N", __("notice"), $msg, 'K');
		am_log($msg);
	}
	return $queue_data;
}

?>