<?php

/*****************************
* Copyright 2021 1st Source IT, LLC
* All rights reserved.
* Permission granted for use as
* long as this copyright notice, associated text and
* links remain in tact.
* Licensed for a single domain and a single instance of EZ-cart.
* Additional licenses can be purchased for additonal sites.
*
* http://www.ez-ms.com
* http://www.ez-om.com
* http://www.1sit.com*
*
* End copyright notification
*/
if( !defined('BOOTSTRAP') ) die('Access denied');

use Tygh\Registry;

/*
** Form extensions
*/
function am_get_form_page_ids() {
	$company_id = am_company_id();
	// 4/7/16: Change to support forms created as company_id 1 and then shared.....
	return db_get_fields("SELECT page_id FROM ?:pages WHERE page_type=?s AND (company_id=0 OR company_id=?i)", 'F', $company_id ? $company_id : 1);
}

function am_page_name($page_id, $lang_code=CART_LANGUAGE) {
	return db_get_field("SELECT page FROM ?:page_descriptions WHERE page_id=?i AND lang_code=?s", $page_id, $lang_code);
}

function am_get_forms() {
	static $forms = array();
	if( $forms )
		return $forms;
		
	$page_ids = am_get_form_page_ids();
//am_dbg("page_ids:".print_r($page_ids,true), true);
	$forms = array();
	foreach($page_ids as $page_id) {
		list($elements, $form_info) = fn_get_form_elements($page_id);
		$email_field = false;
		foreach($elements as $k => $v) {
			if( $v['element_type'] == 'Y' ) {
				$email_field = $k;
			} elseif( strpos($v['description'], 'Email') === 0 
								|| strpos($v['description'], 'E-mail') === 0 
								|| strpos($v['description'], __("email")) === 0) {
				$email_field = $k;
			}
		}
		$page_name = am_page_name($page_id);
		$forms[$page_id] = array('page_name'=>$page_name, 'email_field'=>$email_field, 'elements'=>$elements);
	}
	return $forms;
}

function am_forms_menu($show_select=true) {
	$forms = am_get_forms();
	$menu = array();
	if( $show_select ) {
		$menu[0] = "Select form";
	}
	foreach($forms as $pg_id => $f_info) {
		$menu[$pg_id] = $f_info['page_name'];
	}
	return $menu;
}
	

function am_form_data($page_id, $form_data) {
	$am_data = array();
	$forms = am_get_forms();
	$form = empty($forms[$page_id]) ? array() : $forms[$page_id];
	if( !empty($form['elements']) && $form_data) {
		$am_data['page_id'] = $page_id;
		$am_data['page_name'] = $form['page_name'];
		$am_data['email'] = empty($form_data[$form['email_field']]) ? 'unknown@example.com' : $form_data[$form['email_field']];
		foreach($form_data as $k => $v) {
			if( !empty($v) ) {
				switch($form['elements'][$k]['element_type']) {
					case FORM_DATE:
						$v = fn_parse_date($v);
						break;
					case FORM_REFERER:
						$v = $_SESSION['auth']['referer'];
						break;
					case FORM_IP_ADDRESS:
						$ip = fn_get_ip();
						$v = empty($ip['host']) ? 'unknown' : $ip['host'];
						break;
				}
					
				$am_data['form_fields'][$form['elements'][$k]['description']] = $v;
//				$am_data[$k] = $v['value'];
			}
		}
	}

	return $am_data;
}

?>