<?php
/*****************************
* Copyright 2021 1st Source IT, LLC
* All rights reserved.
* Permission granted for use as
* long as this copyright notice, associated text and
* links remain in tact.
* Licensed for a single domain and a single instance of EZ-cart.
* Additional licenses can be purchased for additonal sites.
*
* http://www.ez-ms.com
* http://www.ez-om.com
* http://www.1sit.com*
*
* End copyright notification
*/

if ( !defined('BOOTSTRAP') )  die('Access denied');
use Tygh\Registry;
use Tygh\Languages;
use Tygh\Languages\Po;
use Tygh\Mailer;
use Tygh\Settings;

// Events
define('ORDER_EVENT', 'O');
define('PROFILE_EVENT', 'P');
define('CART_EVENT', 'C');
define('FORM_EVENT', 'F');	// form extension

define('AM_QUEUE_STATUS_PENDING', 'P');
define('AM_QUEUE_STATUS_SENT', 'S');
define('AM_QUEUE_STATUS_FAILED', 'F');

// Event action
define('ORDER_EVENT_PLACED', 'P');
define('ORDER_EVENT_PRODUCTS_PURCHASED', 'R');
define('ORDER_EVENT_PAYMENT_TYPE', 'T');
define('ORDER_EVENT_AMOUNT_GREATER', 'A');
define('ORDER_EVENT_SHIP_METHOD', 'S');

define('PROFILE_EVENT_ADD', 'U');
define('CART_EVENT_ABANDONED', 'C');

define('AM_USER_TYPE_ALL', 'a');

define('AM_CART_BEFORE_INSTALL', 'before_install');

if( !function_exists('ez_fix_path') ) {
	function ez_fix_path($s) {
		if( DIRECTORY_SEPARATOR != '/' )
			$s = str_replace('/', DIRECTORY_SEPARATOR, $s);
		return $s;
	}
}
function am_dbg($s, $pre=false, $level=0) {
	if( $pre ) {
		echo "<pre>$s</pre>";
	} else if( class_exists('ez_log') ) {
		ez_log::add_msg('debug', $s, $level);
	}
	return $s;
}

function am_log($s, $level=0) {
	if( class_exists('ez_log') ) {
		ez_log::add_msg('auto_mail', $s, $level);
	}
	return $s;
}

function auto_mail_install_lang_vars($force=false) {
	include(dirname(__FILE__)."/lib/lang_vars.php");
	$cnt = 0;
	if( !empty($lang_vars) ) {
		$langs = db_get_fields("SELECT lang_code FROM ?:languages");
		if( empty($langs) )
			$langs = array('en');

		foreach($lang_vars as $var => $val) {
			if( $force || (!($cur = db_get_field("SELECT name FROM ?:language_values WHERE name=?s AND lang_code='en'", $var))) ) {
				$cnt++;
				foreach($langs as $lang) {
					$dat = array('lang_code' => $lang, 'name' => $var, 'value' => $val);
					db_query("REPLACE INTO ?:language_values ?e", $dat);
				}
			}
		}
	}
	// If an install or force, create the po file/language variables.
	if( Registry::get('runtime.controller') == 'addons' || $force ) {
		foreach($langs as $lang) {
			auto_mail_create_po_file("var/langs/{$lang}/addons/auto_mail.po", $lang_vars);
		}
	}
	if( $cnt ) {
		$msg = sprintf("%s: Installed %d language variables into %d languages", Registry::get('runtime.controller'), $cnt, count($langs));
		fn_set_notification('N', __("notice"), $msg, 'K');
	}
}

function auto_mail_create_po_file($po_file, $lang_vars) {
	include(($d=dirname(__FILE__))."/lib/lang_vars.php");
	$dx = explode('/', $d);
	$addon = $dx[count($dx)-1];
	$cnt = 0;
	if( empty($lang_vars) ) 
		return;
		// Get existing po file addon entries
	$po_text = auto_mail_get_addon_po_entries($addon, 'en');

	foreach($lang_vars as $var => $val) {
			$po_text .= 
"msgctxt \"Languages::$var\"
msgid \"$val\"
msgstr \"$val\"

";
	}
		
	$res = file_put_contents($po_file, $po_text);
	if( $res === false )
		fn_set_notification("E", __("error"), "Failed to create po file '$po_file': <pre>".print_r(error_get_last(),true)."</pre>", 'K');
}

function auto_mail_get_addon_po_entries($addon_name, $lang_code='en') {
	$values = Po::getValues(sprintf("%slangs/%s/addons/%s.po", Registry::get('config.dir.var'), $lang_code, $addon_name));
	$saved_entries = array();
	foreach($values as $id => $data) {
		if( strpos($id, 'Languages::') === 0 )
			continue;	// Skip languages
		$saved_entries[$id] = $data;
	}
	if( empty($saved_entries)) {	// No existing PO file.  Create the primary entries
		$saved_entries[''] = array(
										'msgid' => '',
										'msgstr' => array(
                    										0 => 'Project-Id-Version: tygh',
                    										1 => 'Content-Type: text/plain; charset=UTF-8',
                    										2 => 'Language-Team: English',
                    										3 => 'Language: en_US'
                    									),
                    					'msgctxt' =>  '',
                    					'id' => '',
                    					'parent' => '',
                    					'section' => '',
                    			  );
		$saved_entries["Addons::name::$addon_name"] = array
        (
            'msgctxt' => "Addons::name::$addon_name",
            'msgid' => "$addon_name",
            'msgstr' => array(0 => "$addon_name"),
            'id' => "$addon_name",
            'parent' => 'name',
            'section' => ''
        );

		$saveed_entries["Addons::description::$addon_name"] = array
        (
            'msgctxt' => "Addons::description::$addon_name",
            'msgid' => "$addon_name description",
            'msgstr' => array(0 => "$addon_name description"),
            'id' => $addon_name,
            'parent' => 'description',
            'section' => ''
        );
	}
	
	$po_text = 
"msgid \"\"
msgstr \"Project-Id-Version: tygh\"
\"Content-Type: text/plain; charset=UTF-8\"
\"Language-Team: English\"
\"Language: en_US\"

";

	foreach($saved_entries as $id => $po_data) {
		if( empty($id) )	// Header
			continue;
		$po_text .=
"msgctxt \"{$po_data['msgctxt']}\"
msgid \"{$po_data['msgid']}\"
msgstr \"{$po_data['msgstr'][0]}\"

";
	}
	

	
	return $po_text;
}

function auto_mail_install($params=array()) {
	$d=dirname(__FILE__);
	$dx = explode('/', $d);
	$addon_name = $dx[count($dx)-1];
	$verbose = !empty($params['verbose']);

	$force = false;
	foreach($params as $k => $v) {
		switch($k) {
			case 'install_templates':
				if( $v ) {
					$repo_dir = "var/themes_repository/responsive/templates/addons/$addon_name";
					if( $verbose ) print( auto_mail_log("Installing templates from $repo_dir")."\n");
					foreach(glob("design/themes/*", GLOB_ONLYDIR) as $dir) {
						if($verbose) print( auto_mail_log("\t$dir/templates/addons/$addon_name")."\n");
						fn_copy($repo_dir, "$dir/templates/addons/$addon_name");
					}
				}
				break;
			case 'force':
				$force = $v;
				break;
		}
	}
	$func = $addon_name."_install_lang_vars";
	$func($force);
	// Clear all the caches
	fn_clear_cache();
	fn_clear_template_cache();
	$func = $addon_name."_log";
	if( $verbose ) die( $func("Clear caches... Done"));
}


// End

// Do the first installation stuff
function auto_mail_install_setup() {
	auto_mail_install();
	$result = addon_install_setup('auto_mail');
	am_check_table_fields();
	return $result;
}

function auto_mail_lang_vars() {
	$lang_vars = array();
	if( file_exists($f = Registry::get('config.dir.addons')."auto_mail/lib/lang_vars.php") )
		include($f);
	return $lang_vars;
}

function auto_mail_hooks() {
	$hooks = array();
	if( file_exists($f = Registry::get('config.dir.addons')."auto_mail/lib/hook_vars.php") )
		include($f);
	return $hooks;
}	// end function 

// Addon specific Hooks and functions go here

function am_company_id() {
	return Registry::get('runtime.forced_company_id') 
			? Registry::get('runtime.forced_company_id')
			: Registry::get('runtime.company_id');
}

function am_action_type_menu($event_type, $status='A') {
	static $cache = array();
	if( isset($cache[$event_type]) )
		return $cache[$event_type];
		
	$ar = db_get_array("SELECT ?:auto_mail_actions.am_action_id, ?:auto_mail_action_descriptions.description AS action_name 
					   FROM ?:auto_mail_actions
					   LEFT JOIN ?:auto_mail_action_descriptions ON ?:auto_mail_action_descriptions.am_action_id=?:auto_mail_actions.am_action_id AND lang_code=?s
					   WHERE ?:auto_mail_actions.am_event_type=?s AND ?:auto_mail_actions.am_status=?s",
					   DESCR_SL, $event_type, $status);
	$action_menu = array();
	if( empty($ar) )
		$ar = array();
	foreach($ar as $action_info) 
		$action_menu[$action_info['am_action_id']] = $action_info['action_name'];
	return $cache[$event_type] = $action_menu;
}

function am_init_events() {
	$sql_data = array(
					Registry::get('config.dir.addons')."/auto_mail/lib/init_events.php",
					Registry::get('config.dir.addons')."/auto_mail/lib/init_event_descriptions.php"
					);
	foreach($sql_data as $f) {
		include($f);
		foreach($queries as $sql)
			db_query($sql);
	}
}

function am_get_actions($status='A') {
	$ar = db_get_hash_multi_array("SELECT ?:auto_mail_actions.*, ?:auto_mail_action_descriptions.description AS action_name 
							FROM ?:auto_mail_actions
							LEFT JOIN ?:auto_mail_action_descriptions ON ?:auto_mail_action_descriptions.am_action_id=?:auto_mail_actions.am_action_id AND lang_code=?s
							WHERE ?:auto_mail_actions.am_status=?s",
							array('am_event_type', 'am_action_id'), DESCR_SL, $status);
	if( empty($ar) )
		$ar = array();
	$menu = array();
	foreach($ar as $action_id => $action_name) 
		$menu[$action_id] = $action_name;
	return $menu;
}

function am_get_event_actions($event_type, $status='A') {
	$actions = am_action_type_menu($event_type, $status);
	$data = array();
	foreach($actions as $action_id => $action_name) {
		$ar = db_get_row("SELECT * FROM ?:auto_mail_actions WHERE am_action_id=?i", $action_id);
		$ar['am_action_name'] = $action_name;
		$data[$action_id] = $ar;
	}
	return $data;
}


function am_event_types() {
	return array(ORDER_EVENT, PROFILE_EVENT, FORM_EVENT, CART_EVENT);		// form extension
}

function am_get_events($event_type, $status='A', $company_id=0, $include_all_stores=false) {
	static $cache = array();
	$company_id = $company_id ? $company_id : am_company_id();
	$company_condition = $company_id && $include_all_stores 
							? "AND (?:auto_mail_events.company_id=$company_id OR ?:auto_mail_events.company_id=0)"
							: "AND ?:auto_mail_events.company_id=$company_id";
							
//	if( isset($cache[$company_id][$event_type]) ) 
//		return $cache[$company_id][$event_type];
		
	$ar = db_get_hash_array("SELECT ?:auto_mail_events.*, ?:auto_mail_action_descriptions.am_action_type, ?:auto_mail_action_descriptions.description AS am_action_name,
								?:auto_mail_event_descriptions.description AS am_event_name
							 FROM ?:auto_mail_events
					  		 LEFT JOIN ?:auto_mail_action_descriptions ON ?:auto_mail_action_descriptions.am_action_id=?:auto_mail_events.am_action_id 
							 	AND ?:auto_mail_action_descriptions.lang_code=?s
					  		 LEFT JOIN ?:auto_mail_event_descriptions ON ?:auto_mail_event_descriptions.am_event_id=?:auto_mail_events.am_event_id 
							 	AND ?:auto_mail_event_descriptions.lang_code=?s
							 WHERE ?:auto_mail_events.am_event_type=?s 
							 		$company_condition
							 ORDER BY ?:auto_mail_events.am_priority DESC", 'am_event_id', 
							 		CART_LANGUAGE, CART_LANGUAGE, $event_type);
	foreach($ar as $event_id => $e_data) {
		if( $status && ($e_data['am_event_status'] != $status) ) {
			unset($ar[$event_id]);
			continue;
		}
		if( !empty($e_data['am_action_data']) ) {
			$ar[$event_id]['am_action_data'] = unserialize($e_data['am_action_data']);
		}
		if( !empty($e_data['am_event_options']) ) {
			$ar[$event_id]['am_event_options'] = unserialize($e_data['am_event_options']);
		}
	}
//	return $cache[$company_id][$event_type] = $ar;
	if( $event_type == CART_EVENT )
		uasort($ar, 'am_cart_age_compare');
	return $ar;
}

function am_cart_age_compare($event1,$event2) {
	if( empty($event1['am_event_options']['abandon_age'])
	|| empty($event2['am_event_options']['abandon_age']) )
		return true;
	$age1 = am_abandon_age_ts($event1['am_event_options']['abandon_age']);
	$age2 = am_abandon_age_ts($event2['am_event_options']['abandon_age']);
	return $age1 < $age2;
}

function am_get_event($event_id) {
	$ar = db_get_row("SELECT ?:auto_mail_events.*, ?:auto_mail_action_descriptions.am_action_type, ?:auto_mail_action_descriptions.description AS am_action_name,
								?:auto_mail_event_descriptions.description AS am_event_name 
					 		 FROM ?:auto_mail_events
					  		 LEFT JOIN ?:auto_mail_action_descriptions ON ?:auto_mail_action_descriptions.am_action_id=?:auto_mail_events.am_action_id 
							 	AND ?:auto_mail_action_descriptions.lang_code=?s
					  		 LEFT JOIN ?:auto_mail_event_descriptions ON ?:auto_mail_event_descriptions.am_event_id=?:auto_mail_events.am_event_id 
							 	AND ?:auto_mail_event_descriptions.lang_code=?s
							 WHERE ?:auto_mail_events.am_event_id=?i", CART_LANGUAGE, CART_LANGUAGE, $event_id);
	if( !empty($ar['am_action_data']) )
		$ar['am_action_data'] = unserialize($ar['am_action_data']);
	if( !empty($ar['am_event_options']) )
		$ar['am_event_options'] = unserialize($ar['am_event_options']);
	return $ar;
}

function am_update_event($event_id, $data) {
	$_data = fn_check_table_fields($data, 'auto_mail_events');
	if( empty($_data['company_id']) )
		$_data['company_id'] = am_company_id();
	if( isset($_data['am_action_data']) )
		$_data['am_action_data'] = empty($_data['am_action_data']) ? '' : serialize($_data['am_action_data']);
	if( !empty($_data['am_event_options']) )
		$_data['am_event_options'] = serialize($_data['am_event_options']);
		
	// 7/17/16: If this is a create, prpagate the description to all languages.
	$create_lang_descriptions = false;
	if( $event_id ) {	// Existing event
		db_query("UPDATE ?:auto_mail_events SET ?u WHERE am_event_id=?i", $_data, $event_id);
	} else { // Create new event
		$event_id = db_query("INSERT INTO ?:auto_mail_events ?e", $_data);
		// 7/16;16: And update all languages
		$create_lang_descriptions = $event_id;
	}
	if( !empty($data['am_event_name']) ) {
		if( $create_lang_descriptions ) {	// All?
			foreach(Registry::get('languages') as $lang_code => $junk) {
				db_query("REPLACE INTO ?:auto_mail_event_descriptions ?e", array('lang_code'=>$lang_code, 'am_event_id'=>$event_id, 'description'=>$data['am_event_name']) );
			}
		} else {	// Just the current language
			db_query("REPLACE INTO ?:auto_mail_event_descriptions ?e", array('lang_code'=>DESCR_SL, 'am_event_id'=>$event_id, 'description'=>$data['am_event_name']) );
		}
	}
	am_log(__FUNCTION__.": ".(!empty($data['am_event_name']) ? $data['am_event_name'] : "event id: $event_id"), logTrace );
	return am_get_event($event_id);
}

function am_delete_event($event_id) {
	$e_info = am_get_event($event_id);
	db_query("DELETE FROM ?:auto_mail_event_descriptions WHERE am_event_id=?i", $event_id);
	db_query("DELETE FROM ?:auto_mail_events WHERE am_event_id=?i", $event_id);
	am_log(__FUNCTION__.": ".(!empty($e_info['am_event_name']) ? $e_info['am_event_name'] : ", event id: $event_id") );
}

function am_get_due_items($time=TIME, $force=false) {
	if( $force )
		$time += 10000000;
	$ar = db_get_hash_array("SELECT * FROM ?:auto_mail_queue WHERE am_queue_status=?s AND am_next_send < ?i", 'am_queue_id', AM_QUEUE_STATUS_PENDING, $time);
	if( $ar ) {
		foreach($ar as $queue_id => $q_info) {
			if( isset($q_info['am_event']) )
				$ar[$queue_id]['am_event'] = unserialize($q_info['am_event']);
		}
	}
	return $ar ? $ar : array();
}

function am_get_queued_items($status=AM_QUEUE_STATUS_PENDING, $start_time=0, $end_time =0, $limit=200) {
	$company_condition = '';
	if( $company_id = am_company_id() )
		$company_condition = " AND company_id=$company_id";
		
	$sql_status = $sql_start = $sql_end = '';
	$where = 'WHERE 1 ';
	if( $status == AM_QUEUE_STATUS_PENDING )
		$limit = 0;		// Always show all pending emails.
	$limit_str = $limit ? " LIMIT $limit " : '';
	if( $status ) 
		$sql_status = " AND am_queue_status='$status' ";
	if( $start_time )
		$sql_start = " AND am_next_send >= $start_time ";
	if( $end_time )
		$sql_end = " AND am_next_send <= $end_time ";
	$where .= $company_condition . $sql_status . $sql_start . $sql_end;
	$order_by = (($start_time && !$end_time) || ($status == AM_QUEUE_STATUS_SENT)) ? "ORDER BY am_sent DESC, am_next_send ASC" : "ORDER BY am_next_send ASC"; 
	$ar = db_get_hash_array("SELECT * FROM ?:auto_mail_queue $where $order_by $limit_str", 'am_queue_id');
	if( $ar ) {
		foreach($ar as $queue_id => $q_info) {
			if( isset($q_info['am_event']) )
				$ar[$queue_id]['am_event'] = unserialize($q_info['am_event']);
		}
	}
	return $ar ? $ar : array();	
}

function am_set_queue_status($queue_id, $status) {
	$q_info = am_get_queued_item($queue_id);
	$time_ar = $status == AM_QUEUE_STATUS_SENT ? array('am_sent'=>TIME) : array();
	am_update_queue($queue_id, array_merge($time_ar, array('am_queue_status'=>$status)));
	am_log(__FUNCTION__.": Status for queue_id=$queue_id ($q_info[am_send_to]) changed to '$status'");
}

function am_delete_queued_item($queue_id) {
	$q_info = am_get_queued_item($queue_id);
	am_log(__FUNCTION__.": queue_id=$queue_id ".(empty($q_info['am_send_to']) ? '' : "($q_info[am_send_to])") );
	db_query("DELETE FROM ?:auto_mail_queue WHERE am_queue_id=?i", $queue_id);
}

function am_get_queued_item($queue_id) {
	$ar =  db_get_row("SELECT * FROM ?:auto_mail_queue WHERE am_queue_id=?i", $queue_id);
	if( isset($ar['am_event']) )
		$ar['am_event'] = unserialize($ar['am_event']);
	return $ar ? $ar : array();
}

function am_update_queue($queue_id, $data) {
	$_data = fn_check_table_fields($data, 'auto_mail_queue');
	if( isset($_data['am_event']) ) {
		$_data['am_event'] = serialize($_data['am_event']);
	}
	if( $queue_id ) {
		db_query("UPDATE ?:auto_mail_queue SET ?u WHERE am_queue_id=?i", $_data, $queue_id);
	} else {
		$queue_id = db_query("INSERT INTO ?:auto_mail_queue ?e", $_data);
	}
	return am_get_queued_item($queue_id);
}

function am_installed_languages() {
	$lang_info = db_get_fields("SELECT lang_code FROM ?:languages");
	if( empty($lang_info) )
		$lang_info = array('en');
	return $lang_info;
}

function am_mail_path($lang_code) {
	return "design/backend/mail/templates/addons/auto_mail/$lang_code";
}


function am_check_template_path($lang_code, $template_name, $company_id=0) {
	if( $template_name == '' )		// nothing setup yet
		return '';
	$v = Registry::get('view');
	$template_dir = am_mail_path($lang_code);
	$path =  "/addons/auto_mail/$lang_code";
	$path = $template_dir;
	if( file_exists($f = "$template_dir/$template_name") ) {
		return $f;
	} else if( file_exists($f = str_replace("$lang_code/", '', "$template_dir/default_template.tpl")) ) {
		return $f;
	}
	am_log(__FUNCTION__.": No template path found for template name '$template_name', template_dir=$template_dir and lang_code=$lang_code");
	return '';
}

function am_get_files($dir, $subdirs=true) {
	$dir = rtrim($dir, '\\/');
	$files = array();
	$dh = opendir($dir);
	while(($f = readdir($dh)) !== false) {
		if( $f == '.' || $f == '..')
			continue;
		$path = "$dir/$f";
		$is_symlink = is_link($path);
		$is_dir = $is_symlink ? is_dir(readlink($path)) : is_dir($path);
		$is_file = $is_symlink ? is_file(readlink($path)) : is_file($path);
		if( $is_dir ) {
			if(!$subdirs)
				continue;
			$files = array_merge($files, ezm_get_files($path) );
		} elseif( $is_file) {
			$files[] = $path;
		}
	}
	closedir($dh);
	return $files;
}

function am_template_menu($event_type, $lang_code=DESCR_SL) {
	static $cache = array();
	if( isset($cache[$event_type][$lang_code]) )
		return $cache[$event_type][$lang_code];
		
	$menu = array();
	$event_name = 'order';
	switch($event_type) {
		case PROFILE_EVENT: $event_name="profile"; break;
		case CART_EVENT: $event_name="cart"; break;
		// form extension
		case FORM_EVENT: $event_name="form"; break;
		case '': $event_name=''; break;
	}
	$vp = Registry::get('view');
	$template_dir = am_mail_path($lang_code);
	if( !is_dir($template_dir) )
		return $cache[$event_type][$lang_code] = array();
	// Filename format looks like <event_name>_template_name.tpl
//	$templates = fn_get_dir_contents($template_dir, $get_dirs=false, $get_files=true, $extension="tpl");
	// 7/13/16: Moved to local version since 4.3.8 broke fn_get_dir_contents()
	$templates = am_get_files($template_dir, $subdirs=false);
	foreach($templates as $v) {
		$v = str_replace($template_dir . '/', '', $v);	// Get rid of the leading path components.
		if( !preg_match(';\.tpl$;', $v) ) {				// Skip non template files
			continue;
		}
		if( ($event_name && !preg_match(';^'.$event_name.'_;', $v)) || strpos($v, '_subj.tpl') )
			continue;
//		$name = preg_replace(";^{$event_name}_;", '', basename($v, ".tpl"));
		$name = $lang_code."/$v";
		$menu[$v] = $name;
	}
	return $cache[$event_type][$lang_code] = $menu;
}

function am_get_product_code($product_id) {
	return db_get_field("SELECT product_code FROM ?:products WHERE product_id=?i", $product_id);
}

// Ensures start of day for the timestamp returned
function am_next_send($days) {	
	$d_str = date("Y-m-d", (SECONDS_IN_DAY * $days) + TIME);
	return strtotime($d_str);
}

function am_event_statuses($company_id=0, $include_all_stores=false) {
	$company_id = $company_id ? $company_id : am_company_id();
	static $s = array();
	if( !empty($s) )
		return $s;
	foreach(am_get_events(ORDER_EVENT, 'A', $company_id, $include_all_stores) as $event_id => $event_info) {
		$s[] = $event_info['am_event_options']['order_status'];
	}
	return $s;
}



function am_run_queue(&$items_to_run) {
	am_log(__FUNCTION__.": Called" );
	foreach($items_to_run as $am_queue_id => $q_data) {
		am_run_queue_item($am_queue_id, $q_data);
	}
}

function am_show_testimonial() {
	$show_testimonial = Registry::get('addons.discussion.home_page_testimonials') != 'D';
	return $show_testimonial ? 'Y' : 'N';
}

function am_run_queue_item($am_queue_id, &$q_data) {
	$event = $q_data['am_event'];
	$view = Registry::get('view');
	$body_template = $mail_to = '';
	$mail_params = array();
	$am_queue_id = $q_data['am_queue_id'];
	$trigger_data = $q_data['am_trigger_data'];
	$company_id = empty($q_data['company_id']) ? 0 : $q_data['company_id'];
	$lang_code = $q_data['lang_code'];
	$mail_params = am_mail_params($q_data);
	if( empty($mail_params) ) {
		fn_set_notification('E', __("error"), __("am_failed_mail"), 'K');
		am_log(__FUNCTION__.": Failed to send queue_id=$am_queue_id (mail_params)");
		am_set_queue_status($am_queue_id, AM_QUEUE_STATUS_FAILED);
		return;
	}
	Mailer::sendMail($mail_params, 'A', $lang_code);
	am_log(__FUNCTION__.": Sent email to '$mail_params[to]'");
//$m = Registry::get('mailer');
//am_dbg("mailer error: $m->ErrorInfo", true);

	// No idea why they don't support CC and BCC in their parameter processing.
	if( $event['am_cc'] ) {
		foreach(explode(',', $event['am_cc']) as $to) {
			unset($mail_params['to']);
			$mail_params['to'] = $to;
			// 6/12/18 - Change to use area 'A'
			Mailer::sendMail($mail_params, 'A', $lang_code);
		}
	}
	am_set_queue_status($am_queue_id, AM_QUEUE_STATUS_SENT);
}

function am_product_thread_id($product_id, $company_id=1) {
	$company_condition = '';
	if( fn_allowed_for('ULTIMATE') )
		$company_condition = sprintf("AND company_id=%d", $company_id);

	return (integer)db_get_field("SELECT thread_id FROM ?:discussion WHERE object_id=?i AND object_type='P' $company_condition", $product_id);
}


function am_get_payment_name($payment_id, $lang_code=CART_LANGUAGE) {
	return db_get_field("SELECT payment FROM ?:payment_descriptions WHERE payment_id=?i and lang_code=?s", $payment_id, $lang_code);
}

function am_get_shipping_name($shipping_id, $lang_code=CART_LANGUAGE) {
	return db_get_field("SELECT shipping FROM ?:shipping_descriptions WHERE shipping_id=?i AND lang_code=?s",
						$shipping_id, $lang_code);
}

function am_get_product_name($pcode, $lang_code=CART_LANGUAGE) {
	return db_get_field("SELECT ?:product_descriptions.product FROM  ?:products
						LEFT JOIN ?:product_descriptions ON ?:product_descriptions.product_id=?:products.product_id AND lang_code=?s
						WHERE product_code=?s", $lang_code, $pcode);
}

function am_set_debug_level($level) {
if( !($row = db_get_row("SELECT * FROM ?:settings_objects WHERE name='auto_mail_debug_level'")) )
		db_query("INSERT INTO ?:settings_objects SET name='auto_mail_debug_level', value=''");
	db_query("UPDATE ?:settings_objects SET value=?s WHERE name='auto_mail_debug_level'", $level);
	if( class_exists('ez_log') )
		ez_log::set_log_level('auto_mail', $level);
	return $level;
}

function am_get_debug_level() {
	return (integer)db_get_field("SELECT value FROM ?:settings_objects WHERE name='auto_mail_debug_level'");
}

function am_debug_menu() {
	return array(logLog=>'General Logging', logTrace=>'Trace', logDetail=>'Detailed');
}

function am_show_log(&$l_buf, $log_lines, $lines=100) {
//	Registry::get('view')->assign('show_lines', $lines);
	$log_data = array();
	if( $l_buf ) {
		$reversed =& $l_buf; // array_reverse($l_buf);
		for($i = 0; $i < $lines && isset($reversed[$i]); $i++)
			$log_data[] = $reversed[$i];
//	$log_data = array_reverse($log_data);
	}
	Registry::get('view')->assign('log_data', implode("\n", $log_data));
}

function am_order_status_menu($include_blank=false) {
	// 4/5/16: Changed to use function rather than direct DB query.  Seems 4.3.6 broke the query!!!!
	$menu = array();
	if( $include_blank )
		$menu = array(""=>"Select Status");
	$menu = array_merge($menu, fn_get_simple_statuses(STATUSES_ORDER));
	return $menu;
}

function am_shipping_menu($include_blank=false, $lang_code=DESCR_SL) {
	$menu = $include_blank ? array(''=>"Select shipping") : array();
	$shippings = db_get_array("SELECT s.shipping_id, sd.shipping FROM ?:shippings AS s
							   LEFT JOIN ?:shipping_descriptions AS sd ON s.shipping_id=sd.shipping_id
							   		AND sd.lang_code=?s
							   WHERE s.status='A'",
							   $lang_code);
	foreach($shippings as $v) {
		$menu[$v['shipping_id']] = $v['shipping'];
	}
	return $menu;
}

function am_table_exists($table) {
	if( strpos($table, "?:") !== FALSE )
		$table = str_replace("?:", DEFAULT_TABLE_PREFIX, $table);
	$ar = db_get_array("SHOW TABLES");
	$tables = array();
	foreach($ar as $row) {
		$tables[] = current($row);
	}
	return in_array($table, $tables);
}

// Verify that tables have the company_id field (V3 did not) and update if needed
function am_check_table_fields() {
	$tables = array('auto_mail_events', 'auto_mail_queue');
	$actions = array('company_id' => "ADD COLUMN company_id mediumint(8) unsigned not null"
					);
	$notify_label = empty($_REQUEST['addon']) ? Registry::get('runtime.controller') : $_REQUEST['addon'];
	foreach($tables as $table) {
		// Note that '?:' doesn't work with the fn_get_table_fields() function....  It assumes cscart_
		$fields = fn_get_table_fields($table);
		foreach($actions as $field => $alter_action) {
			if( !in_array($field, $fields) ) {
				db_query("ALTER TABLE ?:$table $alter_action");
				fn_set_notification("W", $notify_label, $s = "Modified field '$field' for imported V3 table '$table'", 'K');
				am_log(sprintf("%s: %s", __FUNCTION__, $s));
			}
		}
	}
	
	// Check for old template names
	$old_mail_templates = array(
				'order_body_order_placed.tpl' => 'order_placed.tpl',
				'profile_body_welcome.tpl' => 'profile_welcome.tpl'
								);
	$event_types = am_event_types();
	$converted_events = array();
	$converted_queue_count = 0;
	
	foreach($event_types as $event_type) {
		$events = am_get_events($event_type, $status='', am_company_id(), true);
		foreach($events as $am_event_id => $event_data) {
			if( isset($old_mail_templates[$event_data['am_template']]) ) {
				$converted_events[] = sprintf("Converted template for '%s' from '%s' to '%s'",
											  $event_data['description'], $event_data['am_template'],
											  $old_mail_templates[$event_data['am_template']]
											  );
				$event_data['am_template'] = $old_mail_templates[$event_data['am_template']];
				am_update_event($am_event_id, $event_data);
			}
		}
		// $_REQUEST['force_status'] will override this;
		$status_override = empty($_REQUEST['force_status']) ? '' : $_REQUEST['force_status'];
		if( $event_type == AM_QUEUE_STATUS_PENDING 
		   	||  ($status_override && $event_type == $status_override) ) {
			$year_from_now = TIME + (SECONDS_IN_DAY * 365);
			$queued_items = am_get_queued_items($event_type, $from_time=0, $to_time=$year_from_now);
			foreach($queued_items as $queue_id => $q_data) {
				$event_data = $q_data['am_event'];
				$am_event_id = $event_data['am_event_id'];
				if( isset($old_mail_templates[$event_data['am_template']]) ) {
					$converted_queue_count++;
					$q_data['am_event']['am_template'] = $old_mail_templates[$event_data['am_template']];
					am_update_queue($queue_id, $q_data);
				}
			}
		}
	}
	$msg = '';
	if( $converted_events ) {
		$msg .= implode("<br/>", $converted_events);
	}
	if( $converted_queue_count ) {
		$msg .= sprintf("%sUpdated '%d' queued items with new template names", 
						$msg ? '<br/>' : '',
						$converted_queue_count);
	}
	if( $msg )
		fn_set_notifiation('N', $notify_label, $msg, 'K');
	
	// Check for old template path
	if( is_dir($old_dir = ".design/backend/mail/templates/addons/auto_mail") ) {
		fn_copy($old_dir, $new_dir="./design/backend/mail/templates/addons/auto_mail");
		fn_rm($old_dir);
		fn_set_notification('N', $notify_label, "Moved templates from '$old_dir' to '$new_dir'", 'K');
	}
		
	if( is_dir($old_dir = "./design/backend/mail/addons/auto_mail") ) {
		fn_copy($old_dir, $new_dir="./design/backend/mail/templates/addons/auto_mail");
		fn_rm($old_dir);
		fn_set_notification('N', $notify_label, "Moved templates from '$old_dir' to '$new_dir'", 'K');
	}
	
	// Delete old entries (non V4)
	if( am_table_exists("?:ez_common_config") ) {
		db_query("DELETE FROM ?:ez_common_config WHERE cur_ver NOT LIKE '4%'");
	}
	
	// Ensure all description tables have appropriate language entries
	$desc_tables = array('am_action_id'=>'?:auto_mail_action_descriptions', 'am_event_id'=>'?:auto_mail_event_descriptions');
	$cnt = 0;
	foreach($desc_tables as $key_field => $table) {
		foreach(db_get_array("SELECT * FROM $table WHERE lang_code='en'") as $row) {
			foreach(fn_get_translation_languages() as $row['lang_code'] => $junk) {
				if( $row['lang_code'] == 'en' )
					continue;
				if( !db_get_row("SELECT * FROM $table WHERE lang_code=?s AND $key_field=?i", $row['lang_code'], $row[$key_field]) ) {	
					$cnt++;
					db_query("REPLACE INTO $table ?e", $row);
				}
			}
		}
	}
	if( $cnt ) {
		fn_set_notification('N', $notify_label, sprintf("Updated %d language entires in %d tables",
														$cnt, count($desc_tables)), 'K');
	}
	
	// Subscriber support
	if( !am_table_exists("?:auto_mail_subscribers") ) {
		db_query("CREATE TABLE IF NOT EXISTS `?:auto_mail_subscribers` (
		  `email` varchar(50) NOT NULL,
		  `lang_code` char(2) DEFAULT 'en' NOT NULL,
		  `status` char(1) DEFAULT 'Y' NOT NULL,
		  `timestamp` int(11) unsigned NOT NULL,
		  PRIMARY KEY (`email`, `lang_code`)
		) ENGINE=MyISAM DEFAULT CHARSET=utf8");
		fn_set_notification("N", __("notice"), $s="Created table ".DEFAULT_TABLE_PREFIX."auto_mail_subscribers",'K');
			am_log(__FUNCTION__.": $s");
	}
	
	// 12/3/19: Fix to prevent error if 'am_subscribe' field exists.
	$fields = fn_get_table_fields('users');
	if( !in_array('am_subscribe', $fields) )
		db_query("ALTER TABLE `?:users` ADD `am_subscribe` char(1) NOT NULL DEFAULT 'Y'");
	
/*
	// Abandoned carts
	$fields = fn_get_table_fields('user_session_products');
	if( !in_array('am_sent', $fields) ) {
		db_query("ALTER TABLE ADD am_queued mediumint(11) unsigned");
	}
*/

}

function am_discussion_object_by_thread($thread_id) {
	$object = array();
	// 5/30/18 - Change to compare to 'A' from 'Y'
	if( Registry::get('addons.discussion.status') == 'A' )
		$object = db_get_row("SELECT object_type, object_id, type FROM ?:discussion WHERE thread_id = ?i", (integer)$thread_id);
	return $object;
}

function am_discussion_products($products) {
	$discussion_products = array();
	foreach($products as $k => $item) {
		// 11/7/16: Changed to be conditional upon 'type' (D==disabled)
		$r_data = fn_get_discussion($item['product_id'], 'P');
		if( $r_data['type'] != 'D' ) {
			$discussion_products[$k] = $item;
		} else {
			$dat = (empty($item['product_code']) 
					? sprintf("Id=%d", $item['product_id'])
					: sprintf("Code='%s'", $item['product_code'])
					);
			am_log(__FUNCTION__.": Item $dat skipped.  Discussions not enabled.");
		}
	}
	return $discussion_products;
}
// Hooks

function fn_auto_mail_change_order_status($to, $from, &$order_info, $force_notfication) {
	if( am_subscriber_status($order_info['email'], $order_info['lang_code']) != 'Y' ) {
		am_log(__FUNCTION__.": user '$order_info[email]' is unsubscribed skipping dispatch_order for order_id=$order_info[order_id]" );
		return;	// Person has unsubscribed
	}
	if( in_array($to, am_event_statuses($order_info['company_id'], true)) ) {
		am_log(__FUNCTION__.": processing order_id=$order_info[order_id], to=$to, company_id=$order_info[company_id], include_all_stores=true" , logTrace );
		$queue_data = am_dispatch_order_event($order_info['order_id'], $order_info, $to);
		if( AREA == 'A' && !empty($queue_data[0]) ) 
			fn_set_notification('N', __("notice"),
								str_replace('[EVENT]', $queue_data[0]['am_event']['am_event_name'], __("am_event_queued")),
								'K');
	}
}

function fn_auto_mail_pre_place_order(&$cart) {
	$email = trim(empty($cart['email']) ? $cart['user_data']['email'] : $cart['email']);
	$lang_code = CART_LANGUAGE;
	/*
	$status = (empty($cart['user_data']['am_subscribe']) || $cart['user_data']['am_subscribe'] == 'Y')
					? 'Y'
					: 'N';
	*/
	// re-code to use triplet
	$status = empty($cart['user_data']['am_subscribe']) ? 'Y' : $cart['user_data']['am_subscribe'];
	am_put_subscriber($email, $lang_code, $status);
	am_set_user_subscriber($email, $status);
}

function fn_auto_mail_update_profile($action, &$user_data, &$current_user_data) {
	// Add to the subscribers table
	/* Modified 8/16/23 to account for quick registration or other profile updates where am_subscribe is not set
	if( $user_data['am_subscribe'] != 'Y' ) {	
		$subscriber_status = 'N';
	} else {
		$subscriber_status = 'Y';
	}
	*/
	$subscriber_status = empty($user_data['am_subscribe']) ? 'Y' : $user_data['am_subscribe'];
	// End 8/16/23 modification
	
	am_put_subscriber($user_data['email'], CART_LANGUAGE, $subscriber_status);
	
	if( $subscriber_status == 'Y' ) {
		// Now do Auto_mail processing.
		if( $action != 'add' ) 	// not new profile
			return;	
		$u_type_setting = Registry::get('addons.auto_mail.profile_types');
		if( empty($u_type_setting) )
			$u_type_setting = array(AM_USER_TYPE_ALL);
		// 3/29/15 - BUG - change 'type' to 'user_type'
		if( !in_array($user_data['user_type'], $u_type_setting) )
			return;		// Not for this type
		am_dispatch_profile_event($user_data['user_id'], $user_data, $u_type_setting);
	}
}

function am_send_notification($to, $from, $data, $tpl, $company_id, $lang_code=CART_LANGUAGE, $area='A') {
	return Mailer::sendMail(array(
		'to' => $to,
		'from' => $from,
		'data' => $data,
		'tpl' => $tpl,
		'company_id' => $company_id,
	), $area, $lang_code);
	
}

// EZlog initialization
$addon_name = basename(dirname(__FILE__));
$am_log_level = am_get_debug_level();
if( function_exists('ez_log_prefs')
	&& ($log_prefs = ez_log_prefs())
	&& !empty($log_prefs['enabled']) ) {
	if( empty($log_prefs['log_level']) || $am_log_level > $log_prefs['log_level'] )
		$log_prefs['log_level'] = am_get_debug_level();
	ez_log::init($addon_name, empty($log_prefs['log_level']) ? logLog : $log_prefs['log_level'], 
							empty($log_prefs['area']) ? "" : $log_prefs['area'], 
							$dispaly_errors=false);
} elseif( class_exists('ez_log') ) {
	ez_log::init($addon_name, $am_log_level ? $am_log_level : logLog, '', false);
}

$additional_code = array('dispatch_events.php', 'unsubscribe.php', 
						 'abandoned_carts.php', 'forms.php', 'mail_data.php');
foreach($additional_code as $f) {
	$p = Registry::get("config.dir.addons")."/auto_mail/$f";
	if( file_exists($p) )
		include($p);
	else die("Installation error ($f)");
}

?>