<?php
/*****************************
* Copyright 2021 1st Source IT, LLC
* All rights reserved.
* Permission granted for use as
* long as this copyright notice, associated text and
* links remain in tact.
* Licensed for a single domain and a single instance of EZ-cart.
* Additional licenses can be purchased for additonal sites.
*
* http://www.ez-ms.com
* http://www.ez-om.com
* http://www.1sit.com*
*
* End copyright notification
*/
if( !defined('BOOTSTRAP') ) die('Access denied');

use Tygh\Registry;


function am_mail_params(&$q_data) {
	$am_queue_id = $q_data['am_queue_id'];
	$event = $q_data['am_event'];
	$body_template = $mail_to = '';
	$mail_params = array();
	$trigger_data = $q_data['am_trigger_data'];
	$action_type = $event['am_action_type'];
	$company_id = empty($q_data['company_id']) ? 0 : $q_data['company_id'];
	$lang_code = $q_data['lang_code'];
	switch($q_data['am_event_type']) {
		case ORDER_EVENT:
			switch($action_type) {
				case ORDER_EVENT_PLACED:
				case ORDER_EVENT_AMOUNT_GREATER:
				case ORDER_EVENT_PAYMENT_TYPE:
				case ORDER_EVENT_PRODUCTS_PURCHASED:
				case ORDER_EVENT_SHIP_METHOD:
					$body_template = am_check_template_path($lang_code, $event['am_template'], $company_id);
					if( $body_template ) {
						$mail_params['tpl'] = $body_template;
						$o = fn_get_order_info($order_id = $q_data['am_object_id'], true);
						// 6/19/13 - added hook
						fn_set_hook('am_get_order', $o);
						$mail_params['data']['am_order'] = $o;
						if( empty($o) ) {
							return array();
						}
						$mail_params['to'] = $q_data['am_send_to'];
						
						$mail_params['data']['am_user_info'] =  $u_data = ($o['user_id'] ? fn_get_user_info($o['user_id']) : array());
						$trigger_name = "order_id";
						switch($event['am_action_type']) {
							case ORDER_EVENT_AMOUNT_GREATER:
								$trigger_name = "amount";
								break;
							case ORDER_EVENT_PRODUCTS_PURCHASED:
								$trigger_name = "product_code";
								break;
							case ORDER_EVENT_PAYMENT_TYPE:
								$trigger_name = "payment_type";
								$trigger_data = am_get_payment_name($trigger_data, $lang_code);
								break;
							case ORDER_EVENT_SHIP_METHOD:
								$trigger_name = 'shipping_method';
								$trigger_data = am_get_shipping_name($trigger_data, $lang_code);
								break;
						}
						$mail_params['data']['am_trigger_name'] = $trigger_name;
						$mail_params['data']['am_trigger_data'] = $trigger_data;
						$mail_params['data']['order_id'] = $q_data['am_object_id'];
						if( Registry::get('addons.discussion.status') == 'A' ) {
							$discussion_products = array();
							if( !empty($o['products']) 
							&& ($discussion_products = am_discussion_products($o['products'])) ) {
								foreach($discussion_products as $k => $item) {
//	 10/27/15 - changed am_company_id() to $q_data[
//	 This is for compatibility with old way of doing reviews.
									$mail_params['data']['am_discussion_thread_id'][$item['product_id']] = am_product_thread_id($item['product_id'], $company_id);
								}
							}
							$mail_params['data']['discussion_products'] = $discussion_products;
						}
					} else {	// if body_templae
						am_log(__FUNCTION__.": Template '$event[am_template]' not found.");
						return array();
					}
			}	// switch am_action_type
			break;
			
		case PROFILE_EVENT:
			switch($action_type = $event['am_action_type']) {
				case PROFILE_EVENT_ADD:
					$body_template = am_check_template_path($lang_code, $event['am_template'], $q_data['company_id']);
					if( $body_template ) {
						$mail_params['tpl'] = $body_template;
						$mail_params['data']['am_user_info'] = $u_data = $q_data['am_object_id'] 
																	? fn_get_user_info($q_data['am_object_id']) 
																	: array();
						$mail_params['to'] = $q_data['am_send_to'];
						$mail_params['data']['am_trigger_name'] = 'profile_add';
						$mail_params['data']['am_trigger_data'] = $q_data['am_trigger_data'];
					} else {
						am_log(__FUNCTION__.": Template '$event[am_template]' not found.");
						return array();
					}
			}	// switch am_action_type
			break;
			
		case FORM_EVENT:
			switch($action_type = $event['am_action_type']) {
				case FORM_EVENT:
					// 4/7/16: Added body_template assign
					$body_template = am_check_template_path($lang_code, $event['am_template'], $company_id);
					if( $body_template ) {
						$mail_params['tpl'] = $body_template;
						$mail_params['to'] = $q_data['am_send_to'];
						$mail_params['data']['am_trigger_name'] = 'form_submit';
						$mail_params['data']['form_data'] = $q_data['am_event']['am_action_data'];
					} else {
						am_log(__FUNCTION__.": Template '$event[am_template]' not found.");
						return array();
					}
					break;
			}	// swtich am_action_type
			break;
		
		case CART_EVENT:
			switch($action_type = $event['am_action_type']) {
				case CART_EVENT_ABANDONED:
					$body_template = am_check_template_path($lang_code, $event['am_template'], $company_id);
					if( $body_template ) {
						if( empty($event['am_action_data']) ) {
							fn_set_notification('E', fn_get_lang_var('auto_mail'), "No session_id in am_action_data, send mail failed.", 'K');
							am_set_queue_status($am_queue_id, AM_QUEUE_STATUS_FAILED);
							return array();
						}
						$mail_params['tpl'] = $body_template;
						$cart = am_session_cart($event['am_action_data']);
						
						// Get the image pairs for the product(s)
						if( !empty($cart['products']) ) {
							foreach($cart['products'] as $cid => $p) {
								if( empty($p['main_pair']) )
									$cart['products'][$cid]['main_pair'] = fn_get_image_pairs($p['product_id'], 'product', 'M', true, true, $cart['lang_code']);
							}
						}
						
						fn_set_hook('am_get_cart_contents', $cart);
						// Move the cart data to am_cart
						$mail_params['data']['am_cart'] = $cart;
						if( empty($mail_params['data']['am_cart']) ) {
							fn_set_notification('E', __("error"), "Cart for session_id=$event[am_action_data] no longer exists.  Can't send mail.", 'K');
							return array();
						}
						
						$mail_params['to'] = $q_data['am_send_to'];				
						$mail_params['data']['am_user_info'] =  $u_data = ($cart['user_id'] ? fn_get_user_info($cart['user_id']) : array());
						$trigger_name = __("am_cart_age");
						$mail_params['data']['am_trigger_name'] = $trigger_name;
						// Reset to the age used for this event
						$mail_params['data']['am_trigger_data'] = $event['am_event_options']['abandon_age'];
					} else {
						am_log(__FUNCTION__.": Template '$event[am_template]' not found.");
						return array();
					}
			}	// switch am_action_type
			break;
	}	// switch event_type
	if( $mail_params ) {
		$mail_params['reply-to'] = $mail_params['from'] = 'company_orders_department';
		// 6/12/18 - Add bugfix for include templateDir problem for mailer.
		$mail_params['data']['include_bugfix'] = 'design/backend/mail/templates/';
		if( $action_type == PROFILE_EVENT_ADD ) 
			$mail_params['reply-to'] = $mail_params['from'] = 'company_users_department';
		$mail_params['data']['am_queue_id'] = $am_queue_id;
		$mail_params['company_id'] = $q_data['company_id'];
		$mail_params['data']['show_testimonial'] = am_show_testimonial();
		$mail_params['tpl'] = $body_template;
	}
	return $mail_params;
}
?>